;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of CPCtelera: An Amstrad CPC Game Engine
;;  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.module game

;; void asm_drawSpriteClipped(u16 width_to_draw, void *sprite, u8 width, u8 height, void* memory) __z88dk_callee;
_asm_drawSpriteClipped::
   ;; Get Parameters from the stack 
   pop  hl                       ;; [3] HL = Return Address
   pop  de                       ;; [3] DE = Nothing/Sprite width_to_draw
   ld   a, e                     ;; [1] A = E (Sprite width_to_draw)
   pop  de                       ;; [3] DE = Source pointer (Sprite data array)
   pop  bc                       ;; [3] BC = Height/Width (B = Height, C = Width) 
   ex  (sp), hl                  ;; [6] HL = Destination pointer (Video mem) <-> (SP) Returning back address because __z88dk_callee convention  
   
   ld  (width_to_draw), a        ;; [4] Store Sprite width_to_draw at placeholder    
   neg                           ;; [1] Compute next pixel line Lsb (0x0800 - width_to_draw = 0x07LL)
   ld  (offset_to_next_line), a  ;; [4] Store A to Lsb offset for next line placeholder
   add  c                        ;; [1] A = Sprite Offset (Sprite Width (C) - Sprite Width_to_draw (A))
   ld  (offset_sprite), a        ;; [4] Store Sprite offset at placeholder  
   ld   a, b                     ;; [1] A = B (Sprite height)
   
;; Draw partial Sprite loop
draw_clip_loop:
   ex  de, hl                    ;; [1] HL and DE are exchanged every line to do 16-bits maths with DE        
 
;; Placeholder for the Sprite width_to_draw   
width_to_draw = .+1     
   ld   bc, #0000                ;; [3] BC = Sprite Width to draw  = 0x00XX (XX is a placeholder to be modified) 
   ldir                          ;; [6*C-1] Copy one whole line of bytes from sprite to video memory
   dec  a                        ;; [1] A-- One less iteration to complete Sprite Height
   ret  z                        ;; [2/3] If 0 we have finished the last sprite line and return to caller

;; Placeholder for the Video next line
offset_to_next_line = .+1
   ld   bc, #0x0700              ;; [3] BC = 0x07XXS (XX is a placeholder) = 0x800 bytes is the distance in memory from one pixel line to the next within every 8 pixel lines   
   ex   de, hl                   ;; [1] DE has destination, but we have to exchange it with HL to be able to do 16-bits maths
   add  hl, bc                   ;; [3] | And we add 0x800 minus the width of the sprite (BC) to destination pointer
   ex   de, hl                   ;; [1] DE has destination, but we have to exchange it with HL to be able to do 16-bits maths   

;; Placeholder for the Sprite offset
offset_sprite = .+1      
   ld   bc, #0000                ;; [3] BC = Sprite Offset = 0x00XX (XX is a placeholder to be modified) 
   add  hl, bc                   ;; [3] Add offset sprite    
   ex   de, hl                   ;; [1] DE has destination, but we have to exchange it with HL to be able to do 16-bits maths

;; Sprite 8-bits boundary crossed 	 
   ld   b, a                     ;; [1] Save A (remaing Sprite Height to draw) into B before using it
   ld   a, h                     ;; [1] We check if we have crossed video memory boundaries (which will happen every 8 lines). 
   and  #0x38                    ;; [2] leave out only bits 13,12 and 11 from new memory address (00xxx000 00000000)
   ld   a, b                     ;; [1] Recover A from B
   jp   nz, draw_clip_loop       ;; [2/3]  and checking the 4 bits that identify present memory line. 

   ld   bc, #0xC050              ;; [3] Advance destination pointer to next line
   add  hl, bc                   ;; [3] HL += BC (0xC050)
   jp   draw_clip_loop           ;; [3] Jump to continue with next pixel line