;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of CPCtelera: An Amstrad CPC Game Engine
;;  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.module game

;; void ShiftSpritePixelsRight(u8* sprite, u16 size)  __z88dk_callee
;; Adapted from cpctelera example -move1pixel-
_asm_shiftSpritePixelsRight::
   ;; Get Parameters from the stack 
   pop  hl              ;; [3] HL = Return Address
   pop  de              ;; [3] DE = Sprite
   ex  (sp), hl         ;; [6] HL = Sprite Size (W*H) <-> (SP) Returning back address because __z88dk_callee convention  
   
   ;Fast 16-bits counter
   dec  hl
   ld   b, l
   inc  b
   inc  h
   
   ;u8 prev_rightpixel = 0;
   ld   c, #0x00

   ;do {
shift_right:	
   ;u8 rightpixel = *sprite & 0b01010101;
   ld   a, (de)
   and  #0x55
   ld   l, a

   ;*sprite = (prev_rightpixel << 1) | ((*sprite & 0b10101010) >> 1);
   sla	c
   ld   a, (de)
   and	#0xaa
   srl	a
   or	c
   ld	(de), a
	
   ;prev_rightpixel = rightpixel;
   ld	c, l
	
   ;sprite++;
   inc	de
   
   ;} while (--size);
   djnz shift_right
   dec  h
   jr	nz, shift_right
   ret

;; Macro: ld__ixl_a
;;    Opcode for "LD ixl, a" instruction
;; 
.macro ld__ixl_a
   .dw #0x6FDD  ;; Opcode for ld ixl, a
.endm

;; Macro: ld__a_ixl
;;    Opcode for "LD A, ixl" instruction
;; 
.macro ld__a_ixl
   .dw #0x7DDD  ;; Opcode for ld a, ixl
.endm

;; Macro: or__ixl
;;    Opcode for "OR ixl" instruction
;; 
.macro or__ixl
   .dw #0xB5DD  ;; Opcode for or ixl
.endm
   
;; void ShiftSpritePixelsLeft(u8* sprite, u16 size)  __z88dk_callee
;; Adapted from cpctelera example -move1pixel-
_asm_shiftSpritePixelsLeft::
   ;; Get Parameters from the stack 
   pop  hl              ;; [3] HL = Return Address
   pop  de              ;; [3] DE = Sprite
   ex  (sp), hl         ;; [6] HL = Sprite Size (W*H) <-> (SP) Returning back address because __z88dk_callee convention     
   
   ld  (end_restore_ix + 2), ix  ;; [6] Save IX to restore it before returning
   
   ;u8* next_byte = sprite + 1;
   ld   b, d
   ld   c, e
   inc  bc
   
   ;while (--size) 
shift_left:	   
   dec  hl
   ld   a, h
   or   l
   jp   z, end_shift_left
   
   ;*sprite = ((*sprite & 0b01010101) << 1) 
   ld   a, (de)
   and  a, #0x55
   add  a, a
   ld__ixl_a
   
   ; | ((*(next_byte) & 0b10101010) >> 1); 
   ld   a, (bc)
   and  a, #0xaa
   srl  a
   or__ixl
   
   ld   (de), a
   
   ;sprite++; 
   inc  de
   ;next_byte++;
   inc  bc
   
   jp   shift_left

   ;*sprite = (*sprite & 0b01010101) << 1;
end_shift_left:
   ld__a_ixl
   ld  (de), a   

end_restore_ix:
   ld   ix, #0000                ;; [4] Restore IX before returning   
   ret
