;;-----------------------------LICENSE NOTICE------------------------------------
;;  Deepers Warrens
;;  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;; Credits:
;;    Original routine optimized by @Docent and discussed in CPCWiki :
;; http://www.cpcwiki.eu/forum/programming/cpctelera-colorize-sprite/
;;
;; Thanks to all of them for their help and support.
;;-------------------------------------------------------------------------------
.module assembly

dc_mode0_colorize:: .db 0x00, 0x40, 0x04, 0x44, 0x10, 0x50, 0x14, 0x54, 0x01, 0x41, 0x05, 0x45, 0x11, 0x51, 0x15, 0x55

;; Macro to convert Pixel to xAxC xBxD format
.macro convertPixel              
	;; From cpct_px2byteM0
	ld   bc, #dc_mode0_colorize  ;; [3] BC points to conversion table (dc_mode0_ct)
	
	;; Compute BC += A
	add  c                 ;; [1] | C += A
	ld   c, a              ;; [1] |
	ld   a, #00            ;; [2] A = 0 (preserving Carry Flag)
	adc  b                 ;; [1] | B += Carry
	ld   b, a              ;; [1] |

    ;; A = *(BC + A)
    ld   a, (bc)           ;; [2] A = Value stored at the table pointed by BC
.endm

;; Macro: ld__a_ixl
;;    Opcode for "LD A, ixl" instruction
;; 
.macro ld__a_ixl
   .dw #0x7DDD  ;; Opcode for ld a, ixl
.endm

;; Macro: dec__ixh
;;    Opcode for "DEC IXH" instruction
;; 
.macro dec__ixh
   .dw #0x25DD  ;; Opcode for dec ixh
.endm

;; Macro: dec__ixl
;;    Opcode for "DEC ixl" instruction
;; 
.macro dec__ixl
   .dw #0x2DDD  ;; Opcode for dec ixl
.endm

_asm_spriteColorize::
	;; GET Parameters from the stack 
	ld (sprite_colorize_restore_ix + 2), ix  ;; [6] Save IX to restore it before returning
	pop   hl                     ;; [3] HL = Return Address
	pop   de                     ;; [3] DE = (D = newColor, E = oldColor)
	pop   ix                     ;; [4] IX = Width (IXL) and Height (IXH)

	ld__a_ixl					 ;; [2] A = IXL (Sprite Width)
	ld (convertLoop + 1), a  	 ;; [4] Save A (Sprite Width) in placeholder convertLoop
	
	;; Convert newColor to pixel format (D -> B)
	ld a, d                      ;; [1]  A = D new color index
	convertPixel	             ;; [10] | Convert into A
	ld d, a                      ;; [1]  | D = A new color      : xAxC xBxD

	;; Convert oldColor to pixel format (E -> C)
	ld a, e                      ;; [1]  A = E old color index
	convertPixel	             ;; [10] | Convert into A
	ld c, a                      ;; [1]  | C = A old color      : xAxC xBxD		 
    ld b, d	                     ;; [1]  B = D 
	
	pop   de                     ;; [3] DE = Sprite
	push  hl             		 ;; [4] HL = Returning back address in the stack because function uses __z88dk_callee convention
	
convertLoop:	
	ld  h, #00                  ;; [3] H = #XX (Sprite Width)
	
lineLoop:
	ld	a, (de)                  ;; [2] A = (HL) current Byte of sprite
	and #0x55                    ;; [2] A &= Mask (0x55)             : xBxB xBxB
	
	cp  c                        ;; [1] Test if pixel (A) is the old colour to be replaced (C)
	jr 	nz, readPixelA           ;; [2/3] If not equal go to next pixel 
		ld  a, b                 ;; [1] else A = new colour to set (B)

;; Pixel Mode 0 = ABAB ABAB
readPixelA:
	ld  l, a                     ;; [4] L = A (Save pixel value)
	ld	a, (de)                  ;; [2] A = (HL) current Byte of sprite : ABAB ABAB
	rrca                         ;; [1] A (current byte of sprite) >> 1  : ABAB ABAB -> xABA BABA
	and #0x55                    ;; [2] A &= Mask (0x55)                 : xAxA xAxA
	
	cp  c                        ;; [1] Test if pixel (A) is the old colour to be replaced (C)
	jr 	nz, readPixelB           ;; [2/3] If not equal go to next pixel 
        ld  a, b                 ;; [1] else A = new colour to set (B)
	
readPixelB:
	rlca                         ;; [1] A = xAxA xAxA << 1  : AxAx AxAx	
	or  l                        ;; [2] A |= L (xBxB xBxB)  : ABAB ABAB
  
setByte:  
	ld  (de), a                  ;; [2] Update current sprite byte with pixels
	inc  de                      ;; [2] Next byte sprite
	
	dec h                        ;; [2] Decrement H (Width) 
	jr 	nz, lineLoop             ;; [2/3] If Width != 0 goto lineLoop
	
nextLine:	
	dec__ixh                     ;; [2] Decrement IXH (Height) 
	jr 	nz, convertLoop          ;; [2/3] If != 0 goto convertLoop
	
sprite_colorize_restore_ix:
   ld   ix, #0000                ;; [4] Restore IX before returning
   ret                           ;; [3] Return to caller