//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>
#include "../wincpctelera/wincpctelera_helper.h"

/** Structures includes */
#include "../structures/structures.h"
#include "../structures/memMapping.h"

/** Game includes */
#include "interrupt.h"
#include "../input/input.h"
#include "../video/video.h"
#include "../text/text.h"
#include "../ui/uiboard.h"
#include "../player/player.h"
#include "../player/playerMove.h"
#include "../player/playerDraw.h"
#include "../room/room.h"
#include "../world/world.h"
#include "../sound/sound.h"
#include "../tools/tools.h"

/** Sprites includes */
#include "../sprites/functions/packedSprites.h"
#include "../sprites/allSprites.h"

#ifdef WINCPCTELERA
    #include "../sprites/external/board.h"
    #include "../sprites/external/externalBoard.h"
#else
    #include "../sprites/external/boardSpritesSize.h"
    #include "../sprites/external/externalSpritesSize.h"
#endif 

CPCT_ABSOLUTE_LOCATION_AREA(TEMP_LOC_START);

#define SAVED_BOARD_DATA    (GET_PLAYER_LOC_END - EXTERNAL_BOARD_SIZE)

/** Menu */
#include "../ui/menu.h"

extern BOOL gDrawRoomInProgress;

/*****************************************************/
/*													 */
/*	Set interrupt handler				 			 */
/*												     */
/*****************************************************/
void SetInterruptHandler(TInterrupt interruptFunction)
{
    cpct_waitVSYNC();
#ifndef WINCPCTELERA
    __asm
    halt
    halt
    __endasm;
#endif
    cpct_waitVSYNC();
    cpct_setInterruptHandler(interruptFunction);
}

/*****************************************************/
/*												     */
/*	Save board data    								 */
/*												     */
/*****************************************************/
void SaveBoardData()
{
#ifndef WINCPCTELERA
    cpct_memcpy(SAVED_BOARD_DATA, BOARD_SPRITE_LOC_LOAD, EXTERNAL_BOARD_SIZE);
#endif
}

/*****************************************************/
/*													 */
/*	Initial board					  				 */
/*												     */
/*****************************************************/
void InitBoard()
{
#ifndef WINCPCTELERA
    u8* src = SAVED_BOARD_DATA;
#else
    u8* src = G_board;
#endif

    cpct_drawSprite(Decompress(src + G_BOARD_0_INDEX), cpctm_screenPtr(CPCT_VMEM_START, 0, BOARD_Y), BOARD_W, BOARD_H);
    cpct_drawSprite(Decompress(src + G_BOARD_1_INDEX), cpctm_screenPtr(CPCT_VMEM_START, BOARD_W, BOARD_Y), BOARD_W, BOARD_H);

    cpct_drawSolidBox(cpctm_screenPtr(CPCT_VMEM_START, 0, TEXT_Y - 1), 0xFF, BOARD_W, 8);
    cpct_drawSolidBox(cpctm_screenPtr(CPCT_VMEM_START, BOARD_W, TEXT_Y - 1), 0xFF, BOARD_W, 8);

    CloseBoard();
    RedrawBoard();
}

/*****************************************************/
/*												     */
/*	Initialize game									 */
/*												     */
/*****************************************************/
void Initialize()
{
#ifdef WINCPCTELERA
    LoadWorld();
#endif 
    cpct_disableFirmware();
    SetPaletteColor(HW_BLACK);

    SetDefaultTempLocation();
    
    InitPlayerGraphics();
    SaveBoardData();
    InitSound();
    InitSprites();
    InitWorld();
    InitPlayerGame();
    RunMenu();

    gDrawRoomInProgress = TRUE;

    SetInterruptHandler(sInterruptHandler);
    InitBoard();
    EnterRoom();    
}

CPCT_RELOCATABLE_AREA();
