//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>
#include "../wincpctelera/wincpctelera_helper.h"

/** Structures includes */
#include "../structures/structures.h"

/** Game includes */
#include "../video/video.h"
#include "../tools/tools.h"
#include "../room/room.h"
#include "../world/world.h"
#include "../player/player.h"
#include "../sound/sound.h"

CPCT_ABSOLUTE_LOCATION_AREA(0x40);

/** Palette */
#include "../sprites/g_palette.h"

/** External variables */
volatile BOOL gDrawRoomInProgress;

/*****************************************************/
/*													 */
/*	Set game palette								 */
/*												     */
/*****************************************************/
void SetPalette()
{
    cpct_setPalette(g_palette, sizeof(g_palette));
}

/*****************************************************/
/*													 */
/*	Set palette in same color						 */
/*												     */
/*****************************************************/
void SetPaletteColor(u8 color)
{
    u8 i;
    for (i = 0; i <= 16; i++)
        cpct_setPALColour(i, color);
}

/*****************************************************/
/*													 */
/*	Interruption handler							 */
/*												     */
/*****************************************************/
void sInterruptHandler(void)
{
    static u8 sInterrupt = 0;

    if (sInterrupt == 0)
    {
        if (!gDrawRoomInProgress)
        {
            u8 roomId = gRoomId;
            cpct_setVideoMode(0);

            SetPalette();
            if (roomId != TO_CASTLE_ROOM && roomId != SHOP_ROOM && gRoomType == ROOM_FOREST)
                cpct_setPALColour(2, 0x58);
        }
        else
            SetPaletteColor(HW_BLACK);
    }
    else if (sInterrupt == 1)
    {
        cpct_scanKeyboard_i();
        PlaySound();
    }
    else if (sInterrupt == 3)
    {
        if (gRoomType == ROOM_FOREST)
            cpct_setPALColour(2, 0x44);
    }
    else if (sInterrupt == 5)
    {
#ifndef WINCPCTELERA
        cpct_setVideoMode(2);
#endif
        cpct_setPALColour(0, 0x4B);
    }

    if (++sInterrupt == 6)
    {
        sInterrupt = 0;		
    }
}

CPCT_RELOCATABLE_AREA();