//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>

/** Structures include */
#include "../structures/structures.h"
#include "../player/playerMove.h"

/** Input */
#include "input.h"

/*************** Global Keys *************/
const cpct_keyID gKeys[NB_KEYS] = { Key_CursorUp, Key_CursorDown, Key_CursorLeft, Key_CursorRight, Key_Space, Key_Copy };
const cpct_keyID gJoy[NB_KEYS] = { Joy0_Up, Joy0_Down, Joy0_Left, Joy0_Right, Joy0_Fire1, Joy0_Fire2 };

/*****************************************************/
/*													 */
/*	Check input keys								 */
/*												     */
/*****************************************************/
u8 CheckUserInput() 
{
    u8 movePlayer = NO_MOVE;
    u8 actionPlayer = NO_MOVE;

    if (cpct_isKeyPressed(Key_Esc))
        return SUICIDE;

    for (u8 sel = 0; sel < NB_KEYS; sel++)
    {
        BOOL isKeyPress = cpct_isKeyPressed(gKeys[sel]);

        if (!isKeyPress)
            isKeyPress = cpct_isKeyPressed(gJoy[sel]);

        if (isKeyPress)
        {
            if (movePlayer == NO_MOVE && sel <= KEY_RIGHT)
                movePlayer = 1 << sel;

            if (actionPlayer == NO_MOVE)
            {
                if (sel == KEY_FIRE)
                    actionPlayer = FIRE;
                else
                if (movePlayer == NO_MOVE && sel == KEY_CHANGE_SEL)
                    actionPlayer = SELECT;
            }
        }
    }

    return (movePlayer | actionPlayer);
}

/*****************************************************/
/*													 */
/*	Wait for key released	  	    				 */
/*												     */
/*****************************************************/
void WaitForKeyReleased(u8 key)
{
    while (CheckUserInput() == key);
}

/*****************************************************/
/*													 */
/*	Wait fire button released	  					 */
/*												     */
/*****************************************************/
void WaitForFireReleased()
{
    WaitForKeyReleased(FIRE);
}

/*****************************************************/
/*													 */
/*	Wait select button released	  					 */
/*												     */
/*****************************************************/
void WaitForSelectReleased()
{
    WaitForKeyReleased(SELECT);
}

/*****************************************************/
/*													 */
/*	Wait fire button        	  					 */
/*												     */
/*****************************************************/
void WaitForFire()
{
    while (CheckUserInput() != FIRE);
    WaitForFireReleased();
}