//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>

/** Structures includes */
#include "../structures/structures.h"

/** Decors Sprites */
#include "../sprites/allSprites.h"

/** Video includes */
#include "../video/video.h"
#include "../sprites/functions/packedSprites.h"

/* Game includes */
#include "../room/room.h"
#include "../text/text.h"
#include "../tools/tools.h"

/** Extern variables */
extern SPos gPlayerPos;
extern u8 gTimer;
extern u8 gTempoMonster;

/** Local defines */
#define DEMON_X				9
#define DEMON_Y				90
#define DEMON_SKULL_X		(DEMON_X + SKULL_W + 5)
#define DEMON_SKULL_Y		(DEMON_Y - 9)

#define DEMON_PAINT_X		33
#define DEMON_PAINT_Y		75

#define UPDATE_MOVE_DEMON   3

/*****************************************************/
/*													 */
/*	DrawDemon		  								 */
/*												     */
/*****************************************************/
void DrawDemon()
{
    static const i8 sMove[] = { 0, -1, -2, -1, 0, 1, 2, 1 }; 

    if (HasLightInRoom() && ModuloPowerTwo(gTimer, UPDATE_MOVE_DEMON) == 0)
    {
        i8 move = sMove[ModuloPowerTwo(gTempoMonster++, sizeof(sMove))];

        u8* vmem = GetScreenPtr(DEMON_X, move + DEMON_Y);
        DrawMirrorPackedSpriteMasked(G_DEMON_INDEX, vmem, DEMON_W, DEMON_H);

        vmem = GetScreenPtr(DEMON_SKULL_X, move + DEMON_SKULL_Y);
        DrawPackedSpriteMaskedFlip(G_SKULL_INDEX, vmem, SKULL_W, SKULL_H, FALSE);
    }
}

/*****************************************************/
/*													 */
/*	Get current sprite flame		  				 */
/*												     */
/*****************************************************/
void DrawDemonPaint()
{
    u8 x = gPlayerPos.x;
    if (x < 4 || x > 60)
    {
        u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, DEMON_PAINT_X, DEMON_PAINT_Y);
        SetTempLocation(GET_TEMP_LOC_END - EYES_MOVING_SZ);
        DrawPackedSpriteMaskedFlip(G_EYES_MOVING_INDEX, vmem, EYES_MOVING_W, EYES_MOVING_H, x > 60);
        SetDefaultTempLocation();
    }
}