//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** Player Structure */
#define MAX_INVENTORY		10
#define NOT_FREE_SLOT		-1

#define HAND_LEFT			0
#define HAND_RIGHT			1
#define HANDS				2
#define NOT_IN_HANDS		3

#define PLAYER_WIDTH	    6

typedef struct
{
    BOOL lighted;
    u8 burnout;
} STorch;

typedef struct
{
    u8 inHand[HANDS];
    u8 inventory[MAX_INVENTORY];
    u8 life;
    u16 gold;
} SPlayer;

/** External player */
extern SPlayer gPlayer;
extern u8 gPlayerRoomY;
extern SCoord gPlayerPos;
extern STorch gTorchHand;
extern u8 gCrowBarSolidity;
extern BOOL gAdventure;

void InitPlayerGame();

BOOL IsTorchInHand();
void SetNoTorchInHand();
void ExtinguishTorch();

u8 GetObjectInHand(u8 hand);
u8 IsObjectInHand(u8 itemId);
void InteractPlayer(u8 useItem);
void InteractWithItem(u8* itemPtr, u8 usedItem);
void UseObjectInHand(u8 hand);
void UseCrowBar(BOOL opened);
BOOL UseObjectInInventory(u8 itemId);
void EmptyHand(u8 hand);
i8 PutInInventory(u8 itemId);

void UpdatePlayer();
void UpdateLife(i8 life);
void PlayerDead();
void SetPlayerStartPos();
