//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#pragma once

/** Room draw defines */
#define UPPER_ROOM_Y				0
#define MIDDLE_ROOM_Y				52
#define OUT_ROOM_Y					70

#define FRAME_DOOR_DOWN_H			16
#define POS_FRAME_TOP_Y				2
#define POS_FRAME_DOWN_Y			(ROOM_CY - 3)
#define POS_WALL_LINE_X				2

#define POS_LAT_FRAME_Y				(ROOM_CY - LATT_FRAME_H - 4)
#define POS_LEFT_FRAME_X			0
#define POS_RIGHT_FRAME_X			(SCREEN_CX - LATT_FRAME_W)

#define POS_LAT_DOOR_Y				(POS_LAT_FRAME_Y + 13)
#define POS_LEFT_DOOR_X				(POS_LEFT_FRAME_X + 2)
#define POS_RIGHT_DOOR_X			(POS_RIGHT_FRAME_X + 2)

#define POS_FRONT_DOOR_Y			(POS_FRAME_TOP_Y + 7)
#define POS_FRONT_DOOR_X			(ROOM_CX / 2 - 5)
#define POS_DOOR_SECRET_X			(POS_FRONT_DOOR_X - 1)
#define POS_DOOR_SECRET_Y			(POS_FRONT_DOOR_Y + 24)

#define POS_WALL_FRONT_Y			2
#define POS_STONE_ANGLE_Y			4
#define POS_STONE_ANGLE_X			20
#define STONE_ANGLE_OFFSET			(STONE_ANGLE_H - 6)
#define POS_STONE_ANGLE_R_EXT		68
#define TOP_LINE_HEIGHT				11

#define TORCH_Y						47
#define TORCH_POS					4
#define TORCH_LEFT					TORCH_POS
#define TORCH_RIGHT					(ROOM_CX - TORCH_POS - TORCH_WALL_W)

#define TORCH_FLAME_Y				(TORCH_Y - FLAMES_0_H + 1)
#define TORCH_FLAME_LEFT			(TORCH_LEFT + 1)
#define TORCH_FLAME_RIGHT			(TORCH_RIGHT - 1)

#define GET_LEFT_LIMIT(y)			(20 - y/4)
#define GET_RIGHT_LIMIT(y)			(48 + y/4)

#define POS_LIMIT_TOP				18

// Room
#define ROOM_CX						80
#define ROOM_CY						160

#define ROOM_CORRIDOR_MASK			0b01000000
#define ROOM_PIT_MASK				0b10000000
#define ROOM_LARGE_MASK				0b11000000

#define ROOM_CORRIDOR				1
#define ROOM_PIT					2
#define ROOM_LARGE					3
#define ROOM_MAX_SIZE				3

#define GET_ROOM_SIZE(R)			((0b11000000 & R) >> 6)
#define GET_ROOM_ID(R)				(0b00111111 & R)

// Torch
#define NO_LIGHT					0b00000000
#define ROOM_TYPE					0b10000000
#define ROOM_LIGHT					0b01000000
#define TORCH_LEFT_EMPTY			0b00100000
#define TORCH_RIGHT_EMPTY			0b00010000
#define TORCH_LEFT_EXTING			0b00001000
#define TORCH_RIGHT_EXTING			0b00000100
#define TORCH_LEFT_BURNISH			0b00000010
#define TORCH_RIGHT_BURNISH			0b00000001

#define ROOM_CASTLE					0b00000000
#define ROOM_FOREST					0b10000000

#define HAS_ROOM_DAYLIGHT(T)		((ROOM_LIGHT & T) != 0)
#define HAS_TORCH_SUPPORT_LEFT(T)	(((TORCH_LEFT_EXTING | TORCH_LEFT_BURNISH | TORCH_LEFT_EMPTY) & T) != 0)
#define HAS_TORCH_SUPPORT_RIGHT(T)	(((TORCH_RIGHT_EXTING | TORCH_RIGHT_BURNISH | TORCH_RIGHT_EMPTY) & T) != 0)
#define HAS_TORCH_LEFT(T)			(((TORCH_LEFT_EXTING | TORCH_LEFT_BURNISH) & T) != 0)
#define HAS_TORCH_RIGHT(T)			(((TORCH_RIGHT_EXTING | TORCH_RIGHT_BURNISH) & T) != 0)
#define HAS_TORCH_LEFT_BURNISH(T)	((TORCH_LEFT_BURNISH & T) != 0)
#define HAS_TORCH_RIGHT_BURNISH(T)	((TORCH_RIGHT_BURNISH & T) != 0)
#define HAS_TORCH_BURNISH(T)		(((TORCH_LEFT_BURNISH | TORCH_RIGHT_BURNISH) & T) != 0)
#define GET_ROOM_TYPE(T)			(ROOM_TYPE & T)

// Doors
/** Doors definition */
#define DOOR_WOOD_SOLIDITY			0x02
#define DOOR_PRISON_SOLIDITY		0x03
#define GET_DOOR_SOLIDITY(D)		(0b0000011 & D)
#define SET_DOOR_SOLIDITY(D, S)		((0b11111100 & D) | S)

#define DOOR_NO_WALL				DOOR_TYPE_NO_WALL
#define DOOR_WALL					DOOR_TYPE_WALL
#define DOOR_PASSAGE				DOOR_TYPE_PASSAGE
#define DOOR_WOOD_OPENED			DOOR_TYPE_WOOD | DOOR_LOCK_NO | DOOR_OPENED
#define DOOR_WOOD_CLOSED			DOOR_TYPE_WOOD | DOOR_CLOSED | DOOR_WOOD_SOLIDITY
#define DOOR_WOOD_METAL_KEY			DOOR_TYPE_WOOD | DOOR_LOCK_METAL | DOOR_CLOSED | DOOR_WOOD_SOLIDITY
#define DOOR_WOOD_PRISON_OPENED		DOOR_TYPE_PRISON | DOOR_LOCK_NO | DOOR_OPENED
#define DOOR_WOOD_PRISON_CLOSED		DOOR_TYPE_PRISON | DOOR_CLOSED | DOOR_PRISON_SOLIDITY
#define DOOR_SECRET					DOOR_TYPE_SECRET | DOOR_CLOSED
#define DOOR_SECRET_OPENED			DOOR_TYPE_SECRET | DOOR_LOCK_NO | DOOR_OPENED
#define DOOR_STAIR_DOWN				DOOR_TYPE_STAIR_DOWN | DOOR_LOCK_NO | DOOR_OPENED
#define DOOR_STAIR_UP				DOOR_TYPE_STAIR_UP | DOOR_LOCK_NO | DOOR_OPENED

#define DOOR_TOP					0
#define DOOR_LEFT					1
#define DOOR_RIGHT					2
#define DOOR_DOWN					3

// 0O00 0000
#define DOOR_CLOSED					0b00000000
#define DOOR_OPENED					0b10000000
#define IS_DOOR_OPENED(D)			(DOOR_OPENED & D)

// 00TT T000
#define DOOR_TYPE_WALL				0b00000000
#define DOOR_TYPE_WOOD				0b00010000
#define DOOR_TYPE_PRISON			0b00100000
#define DOOR_TYPE_SECRET			0b00110000
#define DOOR_TYPE_STAIR_DOWN		0b01000000
#define DOOR_TYPE_STAIR_UP			0b01010000
#define DOOR_TYPE_PASSAGE			0b01100000
#define DOOR_TYPE_NO_WALL			0b01110000
#define GET_DOOR_TYPE(D)			(0b01110000 & D)

#define IS_WOOD_DOOR_TYPE(D)	    (D >= DOOR_TYPE_WOOD && D <= DOOR_TYPE_PRISON)

// 0000 LL00 
#define DOOR_LOCK_NO				0b00000000
#define DOOR_LOCK_CROWBAR			0b00000100
#define DOOR_LOCK_METAL				0b00001000
#define DOOR_LOCK_GOLD				0b00001100
#define GET_LOCK_TYPE(L)			(0b00001100 & L)

typedef u8 TDoor; // # o ttt ll ss - Is Open / Type / Lock / Solidity

// DDDD 0000
#define DECO_CENTER_1				0
#define DECO_CENTER_2				1
#define DECO_CENTER_3				2
#define DECO_LAT_LEFT				0
#define DECO_LAT_RIGHT				1

#define NO_DECO						0b00000000

// Limit function
#define LIMIT_LEFT 		0
#define LIMIT_RIGHT		1
#define LIMIT_CX		2

// Nb of items
#define NB_ROOM_ITEMS	3

typedef struct 
{
    u8 type_light_torch;	// thss ttll - Type Room // Highlight Room // Empty / Torch / Lighted Torch     		 
    u8 wallFront[3];	
    u8 doorDef[4];	
} SRoom;

typedef struct
{
    u8 size_roomId;			// ss rrrrrr
    u8 wallLatt[2];
    u8 doorDest[4];
} SRoomStatic;

#define MAX_ITEMS_CHEST	3

// 0000 LL00 
#define CHEST_LOCK_OPEN				0b00000000
#define CHEST_LOCK_LOCKED			0b00000011

typedef struct
{
    u8 roomId;
    u8 solidity;	
    u8 items[MAX_ITEMS_CHEST];
} SChest;

typedef struct
{
    u8 roomId;
    u8 items[NB_ROOM_ITEMS * NB_ROOM_ITEMS]; // Room items
} SItemsRoom;

/** External function */
void DrawTorch();
void DrawRoom();
void CleanRoom(u8 cy);
void InitRooms();
void InitRoomVar();
void ExitRoom(u8 pos);
void EnterRoom();
void UpdateRoom();

void SetRoomDarkness();

BOOL IsRoomLit();
BOOL HasLightInRoom();
BOOL HasRoomWall(u8 roomId, u8 pos);
BOOL HasCurrentRoomWall(u8 pos);
BOOL IsDoorOpened(u8 pos);
void TryOpenLockedDoor(u8 objectId);
void TryOpenSecretPassage();

BOOL ForceChest();
BOOL IsChestOpened();
BOOL IsChestEmpty();

void LightRoomTorch(u8 dir);
void DisplayRoomText();

u8 GetNextRoomIdTo(u8 pos);
u8 GetLimitHoriz(u8 roomY, u8 info);

/** Shared room variables */
extern BOOL _leftWall;
extern BOOL _rightWall;
extern BOOL _topWall;
extern BOOL _downWall;

extern u8 _cyStones;
extern u8 _posLeftAngle, _posRightAngle;
extern u8 _posFrontWall;
extern u8 _startGround;

/** External room variables */
extern u8 gRoomType;
extern u8 gRoomSize;
extern u8 gPrevRoomId;
extern u8 gRoomId;
extern SRoom* gCurrentRoom;
extern u8* gWallLatt;