//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>
#include "../wincpctelera/wincpctelera_helper.h"

/** Structures includes */
#include "../structures/structures.h"

/** Room includes */
#include "room.h"
#include "roomDrawItems.h"

/** Game includes */
#include "../player/player.h"
#include "../video/video.h"
#include "../sprites/functions/moveSprite.h"
#include "../tools/tools.h"
#include "../world/world.h"
#include "../items/torchFlame.h"

/** Decors Sprites */
#include "../sprites/functions/packedSprites.h"
#include "../sprites/allSprites.h"

/** Local defines */
#define STONE_CX		STONE_WALL_W
#define STONE_CY		(STONE_WALL_H/3)
#define STONE_SZ		(STONE_CX*STONE_CY)
#define STONE_LEFT_X    5
#define STONE_RIGHT_X   (STONE_LEFT_X*2 - 1)
#define WALL_WIDTH      4

/** Local structure */
typedef struct
{
    u8 y;
    u8 cx;
    u8 cy;
    u8 spriteOffset;
} SDrawStones;

/*****************************************************/
/*												     */
/*	Draw Top Room lines								 */
/*												     */
/*****************************************************/
void DrawTopRoomLine(BOOL left)
{
    #define POS_LIMIT   (POS_LIMIT_TOP * WALL_LINE_0_H + 2)
    u8 posLimit = POS_LIMIT;

    u8* vmem = left ? cpctm_screenPtr(CPCT_VMEM_START, 0, POS_LIMIT) : cpctm_screenPtr(CPCT_VMEM_START, POS_STONE_ANGLE_R_EXT, POS_LIMIT);
    const u8* sprite = left ? wall_line_2 : wall_line_3;

    u8 x = left ? (ROOM_CX - POS_STONE_ANGLE_R_EXT - 2) : (POS_STONE_ANGLE_R_EXT + 1);
    u8 cx = left ? (ROOM_CX - POS_STONE_ANGLE_R_EXT - 5) : (ROOM_CX - POS_STONE_ANGLE_R_EXT - 2);

    u8 y;
    cpct_memset(vmem, cpctm_px2byteM0(COLOR_BLUE, COLOR_BLUE), cx);

    for (y = 0; y < TOP_LINE_HEIGHT; y++)
    {
        posLimit -= WALL_LINE_0_H;
        vmem = GetScreenPtr(x, posLimit);
        DrawSpriteMasked((u8*)sprite, vmem, WALL_LINE_0_W, WALL_LINE_0_H);

        if (left)
            x += 2;
        else
            x -= 2;
    }
}

/*****************************************************/
/*												     */
/*	Draw Room lines									 */
/*												     */
/*****************************************************/
void DrawRoomLines()
{
    #define POS_BEAM_Y          15
    #define POS_CEIL_Y          31
    #define POS_GROUND_FAR_X    23
    #define POS_GROUND_FAR_Y    38

    u8 x = POS_WALL_LINE_X;
    u8 nbLines = (ROOM_CY - _cyStones * STONE_ANGLE_OFFSET) / WALL_LINE_0_H;

    BOOL left = _leftWall;
    BOOL right = _rightWall;

    if (left || right)
    {
        u8 posY = ROOM_CY;

        u8 y;
        for (y = 0; y < nbLines; y++)
        {
            posY -= WALL_LINE_0_H;

            /** Ceiling */
            if (gRoomSize == ROOM_CORRIDOR)
            {
                u8 ceily = ROOM_CY - posY;

                /** Left */
                if (left)
                {                             
                    u8* vmem = GetScreenPtr(x, ceily);
                    DrawSpriteMasked(wall_line_1, vmem, WALL_LINE_0_W, WALL_LINE_0_H);                    
                }

                /** Right */
                if (right)
                {                  
                    u8* vmem = GetScreenPtr(ROOM_CX - x - 1, ceily);
                    DrawSpriteMasked(wall_line_0, vmem, WALL_LINE_0_W, WALL_LINE_0_H);
                    
                }
            }
            x++;
        }
    }

    /** Top */
    if (_topWall)
    {
        u8 cx = ROOM_CX;
        u8 blanckCx = nbLines + 2;

        if (!left)
            x = 0;
        else
            cx -= blanckCx;

       if (right)
           cx -= blanckCx;

        cpct_memset(GetScreenPtr(x, _startGround), cpctm_px2byteM0(COLOR_BLUE, COLOR_BLUE), cx);

        // Ceiling
        if (gRoomSize == ROOM_CORRIDOR)
        {
            u8* vmem = left ? cpctm_screenPtr(CPCT_VMEM_START, BEAM_W * 2, POS_BEAM_Y) : cpctm_screenPtr(CPCT_VMEM_START, 0, POS_BEAM_Y);
            u8 beamCx = ROOM_CX;
            u8 i;

            if (left)
                beamCx -= 2 * BEAM_W;

            if (right)
                beamCx -= 2 * BEAM_W;

            for (i = 0; i < beamCx / BEAM_W; i++)
            {
                u8* sprite = GetDecompressSpriteFlip(GetPackedSprite((i % 3) ? G_BEAM_0_INDEX : G_BEAM_1_INDEX), BEAM_W, BEAM_H, GetRandBool());
                cpct_drawSprite(sprite, vmem + i * BEAM_W, BEAM_W, BEAM_H);
            }

            if (left)
                DrawPackedSpriteFlip(G_BEAM_BORDER_INDEX, cpctm_screenPtr(CPCT_VMEM_START, BEAM_W * 2, POS_BEAM_Y), BEAM_BORDER_W, BEAM_BORDER_H, TRUE);

            if (right)
                DrawPackedSpriteFlip(G_BEAM_BORDER_INDEX, cpctm_screenPtr(CPCT_VMEM_START, ROOM_CX - BEAM_BORDER_W * 3, POS_BEAM_Y), BEAM_BORDER_W, BEAM_BORDER_H, FALSE);

            cpct_memset(GetScreenPtr(x, POS_CEIL_Y), cpctm_px2byteM0(COLOR_BLUE, COLOR_BLUE), cx);
        }
    }
    else
    {
        u8 roomId = GetNextRoomIdTo(DOOR_TOP);

        if (!left && HasRoomWall(roomId, DOOR_LEFT))
            DrawTopRoomLine(TRUE);

        if (!right && HasRoomWall(roomId, DOOR_RIGHT))
            DrawTopRoomLine(FALSE);

        DrawMirrorPackedSpriteMasked(G_GROUND_FAR_INDEX, cpctm_screenPtr(CPCT_VMEM_START, POS_GROUND_FAR_X, POS_GROUND_FAR_Y), GROUND_FAR_W, GROUND_FAR_H);
    }

    /** Down */
    if (_downWall)
    {
        int i;

        u8 x = left ? 2 : 0;

        u8 cx = ROOM_CX - x;
        if (right)
            cx -= 2;

        for (i = 0; i < WALL_WIDTH; i++)
        {
            u8* vmem = GetScreenPtr(x, ROOM_CY - WALL_WIDTH + i);
            u8 color = cpctm_px2byteM0(COLOR_BLUE, COLOR_BLUE);

            if (i == 1)
                color = cpctm_px2byteM0(COLOR_BRIGHT_CYAN, COLOR_SKY_BLUE);
            else
            if (i == 2)
                color = cpctm_px2byteM0(COLOR_SKY_BLUE, COLOR_BRIGHT_CYAN);

            cpct_memset(vmem, color, cx);
        }
    }
}

/*****************************************************/
/*												     */
/*	Draw Room angle									 */
/*												     */
/*****************************************************/
void DrawRoomAngle(u8 posY, u8 x, u8 cy, BOOL ext, BOOL flip)
{
    u8 index = ext ? 1 : 0;
    u8* angleSprite;
    u8 y;

    if (gRoomSize == ROOM_CORRIDOR)
        index = 2;

    angleSprite = GetDecompressSpriteFlip(GetPackedSprite(G_stone_angle_tileset[index]), STONE_ANGLE_W, STONE_ANGLE_H, flip);

    if (posY != 0)
        DrawSpriteMasked(angleSprite + (STONE_ANGLE_H - posY - 6) * STONE_ANGLE_W, GetScreenPtr(x, 0), STONE_ANGLE_W, posY);

    if (gRoomSize == ROOM_CORRIDOR)
    {
        posY += 2 * STONE_ANGLE_OFFSET;
        cy -= 2;
    }

    for (y = 0; y < cy; y++)
    {
        u8* vmem = GetScreenPtr(x, posY + y * STONE_ANGLE_OFFSET);
        DrawSpriteMasked(angleSprite, vmem, STONE_ANGLE_W, STONE_ANGLE_H);
    }
}

/*****************************************************/
/*												     */
/*	Draw Room angles								 */
/*												     */
/*****************************************************/
void DrawRoomAngles()
{
    if (_topWall)
    {
        if (_leftWall)
            DrawRoomAngle(0, _posLeftAngle, _cyStones, FALSE, TRUE);

        if (_rightWall)
            DrawRoomAngle(0, _posRightAngle, _cyStones, FALSE, FALSE);
    }
    else
    {
        u8 roomId = GetNextRoomIdTo(DOOR_TOP);

        if (!_leftWall && HasRoomWall(roomId, DOOR_LEFT))
            DrawRoomAngle(6, ROOM_CX - POS_STONE_ANGLE_R_EXT - 6, 5, TRUE, FALSE);

        if (!_rightWall && HasRoomWall(roomId, DOOR_RIGHT))
            DrawRoomAngle(6, POS_STONE_ANGLE_R_EXT, 5, TRUE, TRUE);
    }
}

/*****************************************************/
/*												     */
/*	Draw Latteral door								 */
/*												     */
/*****************************************************/
void DrawLatteralDoor(u8 doorPos)
{
    u8 door = gCurrentRoom->doorDef[doorPos];
    if (HaveDoor(door))
    {
        BOOL isDoorRight = (doorPos == DOOR_RIGHT);
        u8 doorType = GET_DOOR_TYPE(door);

        // Draw Door
        u8* vmem = isDoorRight ?	cpctm_screenPtr(CPCT_VMEM_START, POS_RIGHT_DOOR_X, POS_LAT_DOOR_Y) :
                                    cpctm_screenPtr(CPCT_VMEM_START, POS_LEFT_DOOR_X, POS_LAT_DOOR_Y);

        if (doorType == DOOR_TYPE_WOOD)
        {
            DrawPackedSpriteFlip(G_LATT_DOOR_WOOD_INDEX, vmem, LATT_DOOR_WOOD_W, LATT_DOOR_WOOD_H, !isDoorRight);
        }
        else
        {
            cpct_drawSolidBox(vmem, COLOR_BLACK_BYTE, LATT_DOOR_WOOD_W, LATT_DOOR_WOOD_H);

            vmem = isDoorRight ?	cpctm_screenPtr(CPCT_VMEM_START, POS_RIGHT_DOOR_X, POS_LAT_DOOR_Y + LATT_DOOR_WOOD_H - 15) :
                                    cpctm_screenPtr(CPCT_VMEM_START, POS_LEFT_DOOR_X, POS_LAT_DOOR_Y + LATT_DOOR_WOOD_H - 15);

            cpct_memset(vmem, cpctm_px2byteM0(COLOR_SKY_BLUE, COLOR_SKY_BLUE), LATT_DOOR_WOOD_W);
        }

        // Draw Frame
        vmem = isDoorRight ?	    cpctm_screenPtr(CPCT_VMEM_START, POS_RIGHT_FRAME_X, POS_LAT_FRAME_Y) :
                                    cpctm_screenPtr(CPCT_VMEM_START, POS_LEFT_FRAME_X, POS_LAT_FRAME_Y);

        DrawPackedSpriteMaskedFlip(G_LATT_FRAME_INDEX, vmem, LATT_FRAME_W, LATT_FRAME_H, !isDoorRight);
    }
}

/*****************************************************/
/*												     */
/*	Animate open room secret						 */
/*												     */
/*****************************************************/
void AnimationOpenSecretDoor()
{
    if (IsRoomLit() || gTorchHand.lighted)
    {
        u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_DOOR_SECRET_X, POS_DOOR_SECRET_Y);

        u8 i;
        for (i = 1; i < FRONT_DOOR_SECRET_H; i++)
        {
            u8* vmemLine = GetScreenPtr(POS_DOOR_SECRET_X, POS_DOOR_SECRET_Y + i);
            cpct_drawSolidBox(vmemLine, cpctm_px2byteM0(COLOR_BLUE, COLOR_BRIGHT_BLUE), FRONT_DOOR_SECRET_W * 2 - 1, 1);
            DrawMirrorPackedSpriteMasked(G_FRONT_DOOR_SECRET_INDEX, vmem, FRONT_DOOR_SECRET_W, i);
            WinCpcTelera_Wait(50);
        }

        DrawMirrorPackedSpriteMasked(G_FRONT_DOOR_SECRET_INDEX, vmem, FRONT_DOOR_SECRET_W, FRONT_DOOR_SECRET_H);
    }
}

/*****************************************************/
/*												     */
/*	Draw Doors										 */
/*												     */
/*****************************************************/
void DrawDoors()
{
    // Top door
    u8 door = gCurrentRoom->doorDef[DOOR_TOP];
    if (HaveDoor(door))
    {
        u8 doorType = GET_DOOR_TYPE(door);
        u8 posY = (gRoomSize == ROOM_LARGE) ? POS_FRONT_DOOR_Y : POS_FRONT_DOOR_Y + 56;

        // Draw Door
        u8* vmem = GetScreenPtr(POS_FRONT_DOOR_X, posY);
        if (IS_WOOD_DOOR_TYPE(doorType))
        {
            DrawPackedSpriteFlip(G_FRONT_DOOR_WOOD_INDEX, vmem, FRONT_DOOR_WOOD_W, FRONT_DOOR_WOOD_H, FALSE);

            if (doorType == DOOR_TYPE_PRISON)
            {
                vmem = GetScreenPtr(POS_FRONT_DOOR_X, posY + 12);
                DrawMirrorPackedSpriteMasked(G_GRID_DOOR_INDEX, vmem, GRID_DOOR_W, GRID_DOOR_H);
            }
        }
        else if (doorType == DOOR_TYPE_STAIR_DOWN || doorType == DOOR_TYPE_STAIR_UP)
        {
            vmem = GetScreenPtr(POS_FRONT_DOOR_X, posY + 5);
            DrawMirrorPackedSpriteMasked((doorType == DOOR_TYPE_STAIR_DOWN) ? G_STAIR_DOWN_INDEX : G_STAIR_UP_INDEX, vmem, STAIR_DOWN_W, STAIR_DOWN_H);
        }
        else if (doorType == DOOR_TYPE_SECRET)
        {
            if (IS_DOOR_OPENED(door))
            {
                vmem = GetScreenPtr(POS_DOOR_SECRET_X, POS_DOOR_SECRET_Y);
                DrawMirrorPackedSpriteMasked(G_FRONT_DOOR_SECRET_INDEX, vmem, FRONT_DOOR_SECRET_W, FRONT_DOOR_SECRET_H);
            }
        }
        else
        {
            cpct_drawSolidBox(vmem, COLOR_BLACK_BYTE, FRONT_DOOR_WOOD_W, FRONT_DOOR_WOOD_H);

            vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_FRONT_DOOR_X, posY + POS_FRONT_DOOR_Y + 46);
            DrawMirrorPackedSpriteMasked(G_FRONT_DOOR_EMPTY_INDEX, vmem, FRONT_DOOR_EMPTY_W, FRONT_DOOR_EMPTY_H);
        }

        // Draw Frame
        if (doorType != DOOR_TYPE_SECRET)
        {
            posY = (gRoomSize == ROOM_LARGE) ? POS_FRAME_TOP_Y : POS_FRAME_TOP_Y + 56;
            vmem = GetScreenPtr(ROOM_CX / 2 - FRONT_FRAME_W, posY);
            DrawMirrorPackedSpriteMasked(G_FRONT_FRAME_INDEX, vmem, FRONT_FRAME_W, FRONT_FRAME_H);

            if (gRoomId % 3 == 0)
            {
                vmem = GetScreenPtr(ROOM_CX / 2 + FRONT_FRAME_W + SHIELDS_W - 2, posY + 10);
                DrawMirrorPackedSpriteMasked(G_shields_tileset[GetRandMaxTwo()], vmem, SHIELDS_W, SHIELDS_H);
            }
        }
    }

    // Left door
    DrawLatteralDoor(DOOR_LEFT);

    // Right door
    DrawLatteralDoor(DOOR_RIGHT);

    // Down door
    door = gCurrentRoom->doorDef[DOOR_DOWN];

    if (HaveDoor(door))
    {
        // Draw Frame
        u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, (ROOM_CX - DOWN_FRAME_W * 2) / 2, ROOM_CY - DOWN_FRAME_H);
        DrawMirrorPackedSpriteMasked(G_DOWN_FRAME_INDEX, vmem, DOWN_FRAME_W, DOWN_FRAME_H);
    }
}

/*****************************************************/
/*												     */
/*	Draw Torch										 */
/*												     */
/*****************************************************/
void DrawTorch()
{
    u8 torches = gCurrentRoom->type_light_torch;
    u8* vmemLeft = cpctm_screenPtr(CPCT_VMEM_START, TORCH_LEFT, TORCH_Y);
    u8* vmemRight = vmemLeft + TORCH_RIGHT - TORCH_LEFT;

    u8* sprite = GetPackedSprite(G_TORCH_WALL_1_INDEX);

    // Torch
    if (HAS_TORCH_LEFT(torches))
        DrawCompressedSpriteMasked(sprite, vmemLeft, TORCH_WALL_W, TORCH_WALL_H, FALSE);

    if (HAS_TORCH_RIGHT(torches))
        DrawCompressedSpriteMasked(sprite, vmemRight, TORCH_WALL_W, TORCH_WALL_H, TRUE);

    // Support
    sprite = GetPackedSprite(G_TORCH_WALL_0_INDEX);

    if (HAS_TORCH_SUPPORT_LEFT(torches))
        DrawCompressedSpriteMasked(sprite, vmemLeft, TORCH_WALL_W, TORCH_WALL_H, FALSE);

    if (HAS_TORCH_SUPPORT_RIGHT(torches))
        DrawCompressedSpriteMasked(sprite, vmemRight, TORCH_WALL_W, TORCH_WALL_H, TRUE);
}

/*****************************************************/
/*												     */
/*	Draw Torch flame								 */
/*												     */
/*****************************************************/
void DrawTorchFlame(u8 x, u8 flameId)
{
    u8* temp = GET_TEMP_LOC_END - FLAMES_SZ;
    u8* vmem = GetScreenPtr(x, TORCH_FLAME_Y);
    u8* flame = GetFlameSprite(flameId);

    cpct_memset(temp, BACKGROUND_BYTE, FLAMES_SZ);
    cpct_drawToSpriteBufferMaskedAlignedTable(FLAMES_CX, temp, FLAMES_CX, FLAMES_CY, flame, gMaskTable);
    cpct_drawSprite(temp, vmem, FLAMES_CX, FLAMES_CY);
}

/*****************************************************/
/*												     */
/*	Draw Torch flames								 */
/*												     */
/*****************************************************/
void DrawTorchFlames()
{
    u8 torches = gCurrentRoom->type_light_torch;

    if (HAS_TORCH_LEFT_BURNISH(torches))
        DrawTorchFlame(TORCH_FLAME_LEFT, FLAME_TORCH_L);

    if (HAS_TORCH_RIGHT_BURNISH(torches))
        DrawTorchFlame(TORCH_FLAME_RIGHT, FLAME_TORCH_R);
}

/*****************************************************/
/*												     */
/*	Draw Wall Cracks								 */
/*												     */
/*****************************************************/
void DrawWallCracks()
{
    u8* vmem = GetScreenPtr(0, _startGround - WALL_CRACK_H);
    DrawMirrorPackedSpriteMasked(G_WALL_CRACK_INDEX, vmem, WALL_CRACK_W, WALL_CRACK_H);

    if (_leftWall)
        cpct_drawSolidBox(vmem, BACKGROUND_BYTE, _posLeftAngle, WALL_CRACK_H);

    if (_rightWall)
    {
        u8 posRightWallCrack = _posRightAngle + 4;
        cpct_drawSolidBox(vmem + posRightWallCrack, BACKGROUND_BYTE, ROOM_CX - posRightWallCrack, WALL_CRACK_H);
    }
}

/*****************************************************/
/*												     */
/*	Draw Stones				        				 */
/*												     */
/*****************************************************/
void DrawStones(u8 x, u8 cy, i8 offsetx, u8* stone)
{
    u8 posy = GetRandMaxTwo() + (gRoomSize == ROOM_CORRIDOR ? 30 : 0);

    u8 i;
    for (i = 0; i < cy; i++)
    {
        u8 size = STONE_CY;
        u8 posx = x + GetRandMaxTwo() + offsetx;
        u8* vmem = GetScreenPtr(posx, posy);

        if (posy + STONE_CY >= _startGround)
            size = _startGround - posy - 1;

        DrawSpriteMasked(stone + GetRandMaxTwo() * STONE_SZ, vmem, STONE_CX, size);

        posy += STONE_CY + GetRandMaxTwo();
    }
}

/*****************************************************/
/*												     */
/*	Draw Back Stone 								 */
/*												     */
/*****************************************************/
void DrawBackStone(u8 posx, u8 y)
{
    #define NB_BACK_STONES	10

    u8 limit = _rightWall ? _posRightAngle : ROOM_CX;
    u8* stone = GetDecompressPackedSprite(G_STONE_WALL_2_INDEX);

    u8 i;
    for (i = 0; i < NB_BACK_STONES; i++)
    {
        u8 posy = y + GetRandMaxTwo() * STONE_CY + GetRandMaxTwo();

        u8* vmem = GetScreenPtr(posx, posy);
        DrawSpriteMasked(stone + GetRandMaxTwo() * STONE_SZ, vmem, STONE_CX, STONE_CY);

        posx += STONE_CX + GetRandMaxTwo();

        if (posx + STONE_CX > limit)
            break;
    }
}

/*****************************************************/
/*												     */
/*	Draw Back Stones								 */
/*												     */
/*****************************************************/
void DrawBackStones()
{
    u8* stone = GetDecompressPackedSprite(G_STONE_WALL_INDEX);
    u8 offsetx = 0;
    u8 height = 3;

    BOOL hasTwoWalls = _leftWall & _rightWall;

    if (gRoomSize == ROOM_CORRIDOR)
    {
        height = 4;
        offsetx = 4;
    }

    if (_leftWall)
        DrawStones(_posLeftAngle - offsetx, height, STONE_LEFT_X, stone);

    if (_rightWall)
        DrawStones(_posRightAngle + offsetx - 1, height, -STONE_RIGHT_X, stone);

    if (_topWall)
        DrawWallCracks();

    if (!hasTwoWalls)
    {
        u8 x = 0;
        u8 y = 0;

        if (gRoomSize == ROOM_CORRIDOR)
        {
            y = 35;

            if (_leftWall)
                x = 20;
        }
        else
        {
            if (_leftWall)
                x = 40;
        }

        DrawBackStone(x, y);
    }
}

/*****************************************************/
/*												     */
/*	Draw Stones	area								 */
/*												     */
/*****************************************************/
void DrawStoneArea(SDrawStones* drawStone, u8* sprite, BOOL flip)
{
    #define STONE_AREA_CX		LATT_STONE_WALL_W
    #define STONE_AREA_CY		(LATT_STONE_WALL_H / 2)
    #define NB_STONES_AREA		100

    u8 cxStone = drawStone->cx;
    u8 cyStone = drawStone->cy;
    u8 nbStones = (cxStone* cyStone) / NB_STONES_AREA;

    u8 i;
    for (i = 0; i < nbStones; i++)
    {
        u8 posx = GetRandMax(cxStone) / LATT_STONE_WALL_W * LATT_STONE_WALL_W;
        u8 posy = drawStone->y + GetRandMax(cyStone) / LATT_STONE_WALL_H * LATT_STONE_WALL_H;

        if ((posx < 6 && posy > 30) || gRoomSize != ROOM_CORRIDOR)
        {
            u8* vmem = GetScreenPtr(flip ? posx : ROOM_CX - posx - STONE_AREA_CX, posy);
            DrawSpriteMasked(sprite + drawStone->spriteOffset, vmem, STONE_AREA_CX, STONE_AREA_CY);
        }
    }
}

/*****************************************************/
/*												     */
/*	Draw latteral Stones							 */
/*												     */
/*****************************************************/
void DrawLatteralStones(u8 dir)
{
    #define STONE_AREA_SZ	(STONE_AREA_CX*STONE_AREA_CY)
    #define NB_STONES_DRAW  5

    static const SDrawStones sDrawStones[NB_STONES_DRAW] =
    {
        {10, 20, 40, 0 },
        {60, 20, 14, STONE_AREA_SZ},
        {74, 16, 24, STONE_AREA_SZ},
        {98, 10, 24, STONE_AREA_SZ},
        {122, 4, 24, STONE_AREA_SZ}
    };

    BOOL flip = (dir == LEFT);
    u8* stone = GetDecompressSpriteFlip(GetPackedSprite(G_LATT_STONE_WALL_INDEX), LATT_STONE_WALL_W, LATT_STONE_WALL_H, flip);

    const SDrawStones* drawStone = sDrawStones;

    u8 i;
    for (i = 0; i < NB_STONES_DRAW; i++)
        DrawStoneArea(drawStone++, stone, flip);
}

/*****************************************************/
/*												     */
/*	Draw Latteral Ground							 */
/*												     */
/*****************************************************/
void DrawLatteralGround(u8* sprite, u8 hasWall)
{
    u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, 0, OUT_ROOM_Y + 5);

    if (_leftWall == hasWall)
        DrawCompressedSpriteMasked(sprite, vmem, LATT_WALL_W, LATT_WALL_H, FALSE);

    if (_rightWall == hasWall)
        DrawCompressedSpriteMasked(sprite, vmem + SCREEN_CX - LATT_WALL_W, LATT_WALL_W, LATT_WALL_H, TRUE);
}

/*****************************************************/
/*												     */
/*	Draw Room Stones								 */
/*												     */
/*****************************************************/
void DrawWallStones()
{
    if (_topWall)
        DrawBackStones();

    if (_leftWall)
        DrawLatteralStones(LEFT);

    if (_rightWall)
        DrawLatteralStones(RIGHT);
}

/*****************************************************/
/*												     */
/*	Draw Ground										 */
/*												     */
/*****************************************************/
void DrawGround()
{
    u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, 0, OUT_ROOM_Y + 5);

    u8 i;
    for (i = 0; i < GROUND_TILES_NB; i++)
    {
        u8 offset = i * GROUND_W;
        u8* sprite = GetDecompressPackedSprite(G_ground_tileset[i]);
        DrawSpriteMasked(sprite, vmem + offset, GROUND_W, GROUND_H);

        cpct_hflipSpriteM0(GROUND_W, GROUND_H, sprite);
        DrawSpriteMasked(sprite, vmem + (GROUND_TILES_NB + 1) * GROUND_W - offset, GROUND_W, GROUND_H);
    }

    if (_leftWall || _rightWall)
        DrawLatteralGround(GetPackedSprite(G_LATT_WALL_INDEX), TRUE);

    DrawLatteralGround(GetPackedSprite(G_LATT_GROUND_INDEX), FALSE);

    CleanRoom(_startGround);
}

/*****************************************************/
/*												     */
/*	Draw Pit										 */
/*												     */
/*****************************************************/
void DrawPit()
{
    #define POS_PIT_X	(ROOM_CX - 2 * PIT_W) / 2
    #define PIT_CY		(ROOM_CY - PIT_H) / PIT_WALL_H
    
    u8* vmem;
    u8* stone = GetDecompressPackedSprite(G_STONE_WALL_2_INDEX);
    u8 y = 0;

    // Draw stone wall
    u8 total = 0;

    u8 i, j;
    for (i = 0; i < 6; i++)
    {
        u8 posy = i * STONE_CY + GetRandMax(4);
        for (j = 0; j < 8; j++)
        {
            if ((total++ % 3) == 0)
            {
                u8 posx = 2 + j * STONE_CX + GetRandMax(3);
                vmem = GetScreenPtr(posx, posy);
                DrawSpriteMasked(stone + GetRandMaxTwo() * STONE_SZ, vmem, STONE_CX, STONE_CY);
            }
        }
    }

    // Draw pit wall
    for (i = 0; i < PIT_CY; i++)
    {
        vmem = GetScreenPtr(POS_PIT_X, y);
        DrawMirrorPackedSpriteMasked(G_PIT_WALL_INDEX, vmem, PIT_WALL_W, PIT_WALL_H);

        y += PIT_WALL_H;
    }

    vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_PIT_X, ROOM_CY - PIT_H);
    DrawMirrorPackedSpriteMasked(G_PIT_INDEX, vmem, PIT_W, PIT_H);
}

/*****************************************************/
/*												     */
/*	Draw Room in castle								 */
/*												     */
/*****************************************************/
void DrawRoomCastle()
{
    if (HasLightInRoom())
        CleanRoom(ROOM_CY);

    if (gRoomSize != ROOM_PIT)
    {
        DrawGround();
        DrawWallStones();
        DrawRoomLines();
        DrawRoomAngles();
        DrawWallItems();
        DrawRoomItems();
        DrawTorch();
    }
    else
    {
        DrawRoomItems();
        DrawPit();
    }

    DrawDoors();
    
    if (!HasLightInRoom())
        CleanRoom(ROOM_CY);
}
