//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/*************** System includes ***************/
#include <cpctelera.h>

/*************** Sound includes ****************/
#include "sound.h"

/********** Current song & Fx played ***********/
volatile void* _selMusic;
volatile void* _currentMusic;
volatile u8 _currentFx;

/*************** Song & Fx *********************/
#ifndef WINCPCTELERA
    extern void* GAMEMUSIC_START;
    extern void* SOUNDEFFECTS_SOUNDEFFECTS;
#endif

/*****************************************************/
/*													 */
/*	Init SFX							 			 */
/*												     */
/*****************************************************/
void InitSFX(void* sfx_SoundEffects)
{
#ifndef WINCPCTELERA
    PLY_AKM_InitSoundEffects(sfx_SoundEffects);
#endif
    _currentFx = NO_FX;
}

/*****************************************************/
/*													 */
/*	Play SFX							 			 */
/*												     */
/*****************************************************/
void PlaySFX(u8 fx)
{
    _currentFx = fx;
}

/*****************************************************/
/*													 */
/*	Init SFX							 			 */
/*												     */
/*****************************************************/
void ActivateSFX()
{
    if (_currentFx != NO_FX)
    {
#ifndef WINCPCTELERA
        PLY_AKM_PlaySoundEffect(_currentFx, CHANNEL_C, 0);
#endif
        _currentFx = NO_FX;
    }
}

/*****************************************************/
/*													 */
/*	Select new music					 			 */
/*												     */
/*****************************************************/
void InitMusic(void* music)
{
    _selMusic = music;
}

/*****************************************************/
/*													 */
/*	Activate music					 				 */
/*												     */
/*****************************************************/
void ActivateMusic()
{
#ifndef WINCPCTELERA
    if (_currentMusic != _selMusic)
    {
        _currentMusic = _selMusic;
        PLY_AKM_Init(_currentMusic, 0);
    }
#endif
}

/*****************************************************/
/*													 */
/*	Play sound						 				 */
/*												     */
/*****************************************************/
void PlaySound()
{
    ActivateMusic();
    ActivateSFX();

#ifndef WINCPCTELERA
    PLY_AKM_Play();
#endif
}

/*****************************************************/
/*													 */
/*	Init sound						 				 */
/*												     */
/*****************************************************/
void InitSound()
{
#ifndef WINCPCTELERA
    InitSFX(&SOUNDEFFECTS_SOUNDEFFECTS);
    InitMusic(&GAMEMUSIC_START);
#endif
}

/*****************************************************/
/*													 */
/*	Play Select sound				 				 */
/*												     */
/*****************************************************/
void PlaySelect()
{
    PlaySFX(SOUND_SELECT);
}

/*****************************************************/
/*													 */
/*	Play Flame sound				 				 */
/*												     */
/*****************************************************/
void PlayFlame()
{
    PlaySFX(SOUND_FLAME);
}

/*****************************************************/
/*													 */
/*	Play FootStep sound				 				 */
/*												     */
/*****************************************************/
void PlayFootStep()
{
    PlaySFX(SOUND_FOOT_STEP);
}

/*****************************************************/
/*													 */
/*	Play Impact sound				 				 */
/*												     */
/*****************************************************/
void PlayImpact()
{
    PlaySFX(SOUND_IMPACT);
}

/*****************************************************/
/*													 */
/*	Play Door sound				 				     */
/*												     */
/*****************************************************/
void PlayDoor()
{
    PlaySFX(SOUND_DOOR);
}