//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>
#include "../../wincpctelera/wincpctelera_helper.h"

/** Structures includes */
#include "../../structures/structures.h"
#include "../../structures/memMapping.h"

/** Game includes */
#include "../../video/video.h"
#include "../../tools/tools.h"

/* Local variables */
u8* _externSprites;

/** External graphics definition */
#include "../external/externalSpritesSize.h"
#ifdef WINCPCTELERA
    #include "../external/externalGraphics.h"
#endif

/*****************************************************/
/*													 */
/*	Initialize external sprites						 */
/*												     */
/*****************************************************/
void InitSprites()
{
#ifdef WINCPCTELERA
    _externSprites = G_graphics;
#else
    cpct_memcpy((u8*)GRAPHICS_LOC, SPRITES_LOC_LOAD, EXTERNAL_SPRITE_SIZE);
    _externSprites = (u8*)GRAPHICS_LOC;
#endif
}

/*****************************************************/
/*													 */
/*	Get external sprite								 */
/*												     */
/*****************************************************/
u8* GetPackedSprite(u16 offset)
{
    return _externSprites + offset;
}

/*****************************************************/
/*													 */
/*	Get external sprite								 */
/*												     */
/*****************************************************/
u8* GetDecompressPackedSprite(u16 offset)
{
    return Decompress(GetPackedSprite(offset));
}

/*****************************************************/
/*													 */
/*	Draw packed sprite								 */
/*												     */
/*****************************************************/
void DrawPackedSpriteFlip(u16 offset, u8* vmem, u8 cx, u8 cy, BOOL flip)
{
    DrawCompressedSprite(GetPackedSprite(offset), vmem, cx, cy, flip);
}

/*****************************************************/
/*													 */
/*	Draw packed masked sprite						 */
/*												     */
/*****************************************************/
void DrawPackedSpriteMaskedFlip(u16 offset, u8* vmem, u8 cx, u8 cy, BOOL flip)
{
    DrawCompressedSpriteMasked(GetPackedSprite(offset), vmem, cx, cy, flip);
}

/*****************************************************/
/*													 */
/*	Draw mirror packed masked sprite				 */
/*												     */
/*****************************************************/
void DrawMirrorPackedSpriteMasked(u16 offset, u8* vmem, u8 cx, u8 cy)
{
    DrawMirrorCompressedSpriteMasked(GetPackedSprite(offset), vmem, cx, cy);
}	