//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#pragma once

// Inventory
enum
{
    NOTHING = 0,
    GOLDEN_CUP,
    METAL_KEY,
    GOLD_KEY,
    TORCH,
    CROWBAR,
    ROPE,
    HANDLE_OBJECTS = ROPE,
    GOLD_PURSE,		// No in hand			
    POTION,
    LIGHTER,
    RING,
    CROWN,
    SCEPTER,
    SCROLL,
    SCROLL_0 = SCROLL,
    SCROLL_1,
    SCROLL_2,
    OBJECTS,
    CHEST_ACTION
};

// Front wall items
enum
{
    WALL_EMPTY = 64,
    WALL_TAPESTRY_BLUE,
    WALL_TAPESTRY_RED,
    WALL_TAPESTRY_ORANGE,
    WALL_FOREST_SIGN,
    WALL_WINDOW,
    WALL_FIREPLACE,
    WALL_CHAINS,
    WALL_PAINT_0,
    WALL_PAINT_1,
    WALL_PAINT_2,
    WALL_PAINT_3,
    WALL_SHIELD_0,
    WALL_SHIELD_1,
    WALL_SHIELD_2,
    WALL_ALTAR,
    WALL_ALTAR_GOLD_CUP,
    WALL_ALTAR_RING,
    WALL_ALTAR_CROWN,
    WALL_ALTAR_SCEPTER,
    WALL_SECRET_KEY_HOLE,
    WALL_FIREPLACE_KEY,
};

// Room items
enum
{
    ROOM_STATIC = 128,
    ROOM_TABLE = ROOM_STATIC,
    ROOM_CHEST_CLOSE,
    ROOM_CHEST_OPEN,
    ROOM_SKELETON,
    ROOM_DEMON_CIRCLE,
    ROOM_DEMON,
    ROOM_ITEM_TRAP,
    ROOM_ITEM_HIDE_TRAP,
    ROOM_EVENT_LIGHT_OFF
};

/** Adventure status */
#define	ADV_START					0b00000000
#define	ADV_CUP_FILLED			    0b00000001
#define	ADV_DEMON_VANISHED			0b00000010
#define	ADV_CROWN_TAKEN				0b00000100
#define	ADV_RING_TAKEN				0b00001000
#define	ADV_SCEPTER_TAKEN			0b00010000
#define ADV_WIN						(ADV_CROWN_TAKEN | ADV_RING_TAKEN | ADV_SCEPTER_TAKEN)
#define IS_GAME_WIN(X)              ((X & ADV_WIN) == ADV_WIN)

/** Game constants */
#define START_GOLD  	        	50
#define DEMON_DAMAGES               5
#define TORCH_DURATION	            100
#define MAX_LIFE		        	100
#define LIGHTER_NB_USES			    10
#define CROWBAR_NB_USES             20
#define NB_GOLD_COINS				100
#define POTION_EXTRA_LIFE			50
#define FALL_LOSE_LIFE				-30
#define START_POS_X					32
#define START_POS_Y					(5 + 2 * 15)
#define START_ROOM_POS_Y			1
#define SKY_COLOR	    	        cpctm_px2byteM0(COLOR_BRIGHT_CYAN, COLOR_BRIGHT_CYAN)
#define NIGHT_COLOR			        cpctm_px2byteM0(COLOR_BLACK, COLOR_BLACK)

/** Special rooms */
#define ROOM_START				    0
#define TO_CASTLE_ROOM				2
#define LAC_FOREST_ROOM				3
#define DEMON_ROOM_PAINT_1			7
#define DEMON_ROOM_PAINT_2			16
#define DEMON_AREA					10
#define CELL_KING_ROOM				23
#define WIN_ROOM                    29
#define TO_CHAPEL_ROOM				37
#define CHAPEL_ROOM					38
#define DUNGEON_PIT_ROOM			39
#define DEMON_ROOM					43
#define DUNGEON_ROOM				56
#define PIT_ROOM					(MAX_ROOMS - 1)
#define SHOP_ROOM					(MAX_ROOMS + 1)

//#define DEBUG_MODE                  1