//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#pragma once
#include "../sprites/external/externalSpritesSize.h"
#include "../sprites/external/boardSpritesSize.h"
#include "../world/worldSize.h"

/** Firmware 0xA6FC to 0xBFFF */
#define STACK_SIZE                  0xA0
#define VMEM_SIZE                   0x4000
#define VMEM_START                  0xC000

/** Temporary size */
#define TEMP_MEM_SZ                 0x400	// = 1024

/** Mask table location */
#define MASK_TABLE_LOC              0x100

/** Players constant */
#define PLAYER_SPRITE_CX            13
#define PLAYER_SPRITE_CY            (70 + 12) // = Player + Flame
#define PLAYER_SPRITE_SZ            (PLAYER_SPRITE_CX * PLAYER_SPRITE_CY) // = 13*82 = 1066

/** Loading address */
#define SPRITES_LOC_LOAD            CPCT_VMEM_START
#define	BOARD_SPRITE_LOC_LOAD       (CPCT_VMEM_START + EXTERNAL_SPRITE_SIZE)
#define	WORLD_STATIC_LOC_LOAD       (CPCT_VMEM_START + EXTERNAL_SPRITE_SIZE + EXTERNAL_BOARD_SIZE)
#define	WORLD_LOC_COMP_LOAD         (CPCT_VMEM_START + EXTERNAL_SPRITE_SIZE + EXTERNAL_BOARD_SIZE + WORLD_STATIC_SIZE)

/** Memory mapping */
#define FREE_MEM_START              (VMEM_START - STACK_SIZE)

#define	GRAPHICS_LOC                (FREE_MEM_START - EXTERNAL_SPRITE_SIZE) 
#define WORLD_STATIC_LOC            (GRAPHICS_LOC - WORLD_STATIC_SIZE)
#define WORLD_LOC                   (WORLD_STATIC_LOC - WORLD_SIZE)
#define WORLD_LOC_COMPRESS          (WORLD_LOC - WORLD_COMPRESS_SIZE)

#define PLAYER_LOC_END              (WORLD_LOC_COMPRESS - 1)
#define PLAYER_LOC                  (WORLD_LOC_COMPRESS - PLAYER_SPRITE_SZ*2)   // Total temporary memory = TEMP_LOC_START - PLAYER_LOC = 0x0C54 (3156)
#define TEMP_LOC_END                (PLAYER_LOC - 1)
#define TEMP_LOC_START              (PLAYER_LOC - TEMP_MEM_SZ)    

#ifndef WINCPCTELERA
#define GET_TEMP_LOC_START      (u8*)TEMP_LOC_START					
#define GET_TEMP_LOC_END        (u8*)TEMP_LOC_END
#define GET_PLAYER_LOC_START    (u8*)PLAYER_LOC
#define GET_PLAYER_LOC_END		(u8*)PLAYER_LOC_END
#define	GET_WORLD_STATIC_LOC    (SWorldStatic*)WORLD_STATIC_LOC
#define	GET_WORLD_LOC           (SWorld*)WORLD_LOC    
#define	GET_WORLD_LOC_COMPRESS  (u8*)WORLD_LOC_COMPRESS   
#else
#define GET_TEMP_LOC_START      (u8*)WinCpcTelera_GetMemPtr(TEMP_LOC_START)
#define GET_TEMP_LOC_END        (u8*)WinCpcTelera_GetMemPtr(TEMP_LOC_END)
#define GET_PLAYER_LOC_START    (u8*)WinCpcTelera_GetMemPtr(PLAYER_LOC)
#define GET_PLAYER_LOC_END		(u8*)WinCpcTelera_GetMemPtr(PLAYER_LOC_END)
#define	GET_WORLD_STATIC_LOC    (SWorldStatic*)WinCpcTelera_GetMemPtr(WORLD_STATIC_LOC)
#define	GET_WORLD_LOC           (SWorld*)WinCpcTelera_GetMemPtr(WORLD_LOC)
#define	GET_WORLD_LOC_COMPRESS  (u8*)WinCpcTelera_GetMemPtr(WORLD_LOC_COMPRESS)    
#endif

