//-----------------------------LICENSE NOTICE------------------------------------
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>
#include <stdio.h>

/** Game includes */
#include "fontsHelper.h"
#include "text.h"
#include "../structures/structures.h"
#include "../video/video.h"
#include "../player/player.h"

/** Sprites includes */
#include "../sprites/functions/packedSprites.h"
#include "../sprites/allSprites.h"

/** Fonts variables*/
u8 gFontCx;
u8 gFontCy;
const u8* gFonts;

/*****************************************************/
/*													 */
/*	Convert int to string							 */
/*												     */
/*****************************************************/
static void ConvertIntToChar(u16 num, char* str, u8 length)
{
    if (num != 0)
    {
        u8 len = 0;
        u16 n = num;
        u8 start;
        u8 i;

        while (n != 0)
        {
            len++;
            n /= 10;
        }

        start = length - len;

        for (i = 0; i < len; i++)
        {
            str[len - (i + 1) + start] = num % 10 + '0';
            num = num / 10;
        }
    }
    else
    {
        str[length - 1] = '0';
    }

    str[length] = '\0';
}

/*****************************************************/
/*													 */
/*	Display character							 	 */
/*												     */
/*****************************************************/
static void DisplayChar(u8 x, u8 y, char c)
{
    /** Is letter or number character */	
    u8* spriteChara;
    u8 charaIndex;

    if (c == '\'')
        charaIndex = 26;
    else if (c == '#')
        charaIndex = 0;
    else
        charaIndex = (c > 64) ? c - 65 : c - 47;

    spriteChara = gFonts + charaIndex * gFontCx * gFontCy;
    cpct_drawSprite(spriteChara, GetScreenPtr(x, y), gFontCx, gFontCy);
}

/*****************************************************/
/*													 */
/*	Get string length						 		 */
/*												     */
/*****************************************************/
u8 GetStrLength(const char* text)
{
    u8 i = 0;
    while (*text != '\0')
    {
        text++;
        i++;
    }
    return i;
}

/*****************************************************/
/*													 */
/*	Display String							 		 */
/*												     */
/*****************************************************/
void PrintStr(u8 x, u8 y, const char* text)
{
    while (*text != '\0')
    {
        if (*text != ' ')
            DisplayChar(x, y, *text);
            
        x += gFontCx;
        text++;
    }
}

/*****************************************************/
/*													 */
/*	Display Text							 		 */
/*												     */
/*****************************************************/
void PrintText(const char* text)
{
    #define TEXT_BORDER	10	
    
    u8 len = GetStrLength(text);
    u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, TEXT_BORDER, TEXT_Y);
    cpct_drawSolidBox(vmem, 0xFF, SCREEN_CX - TEXT_BORDER * 2, FONT_CY);

    gFonts = G_fonts;
    gFontCx = FONT_CX_TEXT;
    gFontCy = FONT_CY;

    PrintStr((SCREEN_CX - len) / 2, TEXT_Y, text); 
}

/*****************************************************/
/*													 */
/*	Display Integer						 			 */
/*												     */
/*****************************************************/
void PrintInt(u8 x, u8 y, u16 val, u8 length)
{		
    #define STR_LENGTH	5

    /** [0, 65535] */
    static const char sConv[STR_LENGTH];
    cpct_memset(sConv, '#', STR_LENGTH);

    gFonts = GetDecompressPackedSprite(G_NUMBER_INDEX);
    gFontCx = FONT_CX_NUMBER;
    gFontCy = FONT_CY;

    ConvertIntToChar(val, sConv, length);
    PrintStr(x + (STR_LENGTH - length - 1) * FONT_CX_NUMBER, y, sConv);
}