//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>

/** Structures includes */
#include "../structures/structures.h"

/** Game includes */
#include "../video/video.h"
#include "../tools/tools.h"

/** ZX7 includes */
#include "../asm/ZX7Decrunch.h"

/** Temporay location */
static u8* _tempLoc;

/*****************************************************/
/*													 */
/*	Define default tempory location		 			 */
/*												     */
/*****************************************************/
void SetDefaultTempLocation()
{
    SetTempLocation(GET_TEMP_LOC_START);
}

/*****************************************************/
/*													 */
/*	Define tempory location				 			 */
/*												     */
/*****************************************************/
void SetTempLocation(u8* mem)
{
    _tempLoc = mem;
}

/*****************************************************/
/*													 */
/*	Decompress Data									 */
/*												     */
/*****************************************************/
u8* Decompress(const u8* data)
{
    ZX7Decrunch(data, _tempLoc);
    return _tempLoc;
}

/*****************************************************/
/*													 */
/*	Get Random 0 to max								 */
/*												     */
/*****************************************************/
u8 GetRandMax(u8 max)
{
    return cpct_getRandom_xsp40_u8() % (max + 1);
}

/*****************************************************/
/*													 */
/*	Get Random 0 to 2								 */
/*												     */
/*****************************************************/
u8 GetRandMaxTwo()
{
    return GetRandMax(2);
}

/*****************************************************/
/*													 */
/*	Set Random seed									 */
/*												     */
/*****************************************************/
void SetRandSeed(u8 value)
{
    #define MAGIC_RAND_NUMBER_32	0xAAAABBBB
    #define MAGIC_RAND_NUMBER_16	0x00112233
    cpct_setSeed_xsp40_u8(value * MAGIC_RAND_NUMBER_16, MAGIC_RAND_NUMBER_32);
}
