//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------
#include <cpctelera.h>

/** Game includes */
#include "../text/fontsHelper.h"
#include "../items/torchFlame.h"

/** ZX7 includes */
#include "../asm/ZX7Decrunch.h"

/** Sprites includes */
#include "../Sprites/local/fonts_menu.h"
#include "../Sprites/local/icons.h"
#include "../Sprites/local/letters.h"
#include "../Sprites/local/letters_title.h"
#include "../sprites/local/flames.h"

/** Temporary */
u8 _temp[100];

/** External functions */
extern void SetPaletteColor(u8 color);
extern void SetPalette();

/** Defines local */
#define FONT_MENU_CX        2
#define FONT_MENU_CY        6

#define POS_MENU_X		    32
#define POS_MENU_Y		    30

#define POS_VERSION_X		(SCREEN_CX - FONT_MENU_CX * 3)
#define POS_VERSION_Y		(SCREEN_CY - FONT_MENU_CY)

#define POS_CREDITS_Y	    180

#define OFFSET_Y_SELECT     7
#define POS_X_SELECT        POS_MENU_X - 10

#define LETTER_CX           4  
#define LETTER_CY           8
#define LETTER_SZ           (LETTER_CX*LETTER_CY)

#define ICON_CX             4  
#define ICON_CY             8
#define ICON_SZ             (ICON_CX*ICON_CY)

#define POS_TITLE_Y         10
#define POS_THRONE_Y        POS_TITLE_Y
#define POS_THRONE_X        28
#define POS_LEGACY_X        (POS_THRONE_X + 16)
#define POS_LEGACY_Y        (POS_THRONE_Y + 2)

#define POS_START_MENU_Y    (POS_TITLE_Y + 60)
#define POS_REDEF_MENU_Y    (POS_START_MENU_Y + 20)
#define OFFSET_Y_LETTER     2 

#define POS_TEXT_X(T)       (SCREEN_CX - (T * FONT_MENU_CX)) / 2

/** Strings */
static const char REDEF_KEYS_STR[] = "EDEFINE KEYS";
static const char THRONE_STR[] = "HRONE";
static const char LEGACY_STR[] = "EGACY";
static const char START_STR[] = "TART";

/** Redefine keys */
#include "redefine.h"

/*****************************************************/
/*													 */
/*	Init fonts           				 			 */
/*												     */
/*****************************************************/
void InitFonts()
{
    gFonts = fonts_menu;
    gFontCx = FONT_MENU_CX;
    gFontCy = FONT_MENU_CY;
}

/*****************************************************/
/*													 */
/*	Draw title           				 			 */
/*												     */
/*****************************************************/
void DrawTitle()
{
    CleanRoom(SCREEN_CY);
    ZX7Decrunch(G_letters_title, _temp);

    PrintStr(POS_THRONE_X, POS_THRONE_Y, THRONE_STR);
    cpct_drawSprite(_temp, cpctm_screenPtr(CPCT_VMEM_START, POS_THRONE_X - LETTER_CX, POS_THRONE_Y - OFFSET_Y_LETTER), LETTER_CX, LETTER_CY);

    PrintStr(POS_LEGACY_X, POS_LEGACY_Y, LEGACY_STR);
    cpct_drawSprite(_temp + LETTER_SZ, cpctm_screenPtr(CPCT_VMEM_START, POS_LEGACY_X - LETTER_CX, POS_LEGACY_Y - OFFSET_Y_LETTER), LETTER_CX, LETTER_CY);
}

/*****************************************************/
/*													 */
/*	Draw menu           				 			 */
/*												     */
/*****************************************************/
void DrawMenu()
{    
    DrawTitle();

    ZX7Decrunch(G_letters, _temp);

    PrintStr(POS_MENU_X, POS_START_MENU_Y, START_STR);
    cpct_drawSprite(_temp, cpctm_screenPtr(CPCT_VMEM_START, POS_MENU_X - LETTER_CX, POS_START_MENU_Y - OFFSET_Y_LETTER), LETTER_CX, LETTER_CY);

    PrintStr(POS_MENU_X, POS_REDEF_MENU_Y, REDEF_KEYS_STR);
    cpct_drawSprite(_temp + LETTER_SZ, cpctm_screenPtr(CPCT_VMEM_START, POS_MENU_X - LETTER_CX, POS_REDEF_MENU_Y - OFFSET_Y_LETTER), LETTER_CX, LETTER_CY);

    PrintStr(POS_MENU_X, POS_REDEF_MENU_Y, REDEF_KEYS_STR);

    ZX7Decrunch(G_icons, _temp);
}

/*****************************************************/
/*													 */
/*	Init menu              				 			 */
/*												     */
/*****************************************************/
void InitMenu()
{
    InitFonts();

    cpct_setVideoMode(0);
    SetPalette();
    cpct_setPALColour(0, 0x54);

    DrawMenu();
}

/*****************************************************/
/*													 */
/*	Print credit           				 			 */
/*												     */
/*****************************************************/
void PrintCredit(u8 x, const char* credit)
{
    cpct_drawSolidBox(cpctm_screenPtr(CPCT_VMEM_START, 5, POS_CREDITS_Y - 3), cpctm_px2byteM0(COLOR_BLACK, COLOR_BLACK), SCREEN_CX / 2, FONT_MENU_CY + 8);
    cpct_drawSolidBox(cpctm_screenPtr(CPCT_VMEM_START + SCREEN_CX / 2, 5, POS_CREDITS_Y - 3), cpctm_px2byteM0(COLOR_BLACK, COLOR_BLACK), SCREEN_CX / 2, FONT_MENU_CY + 8);
    PrintStr(x, POS_CREDITS_Y, credit);
}

/*****************************************************/
/*													 */
/*	Clean menu selection         		 			 */
/*												     */
/*****************************************************/
void CleanMenuSelection(BOOL selection)
{
    u8* memVideo = selection ? cpctm_screenPtr(CPCT_VMEM_START, POS_X_SELECT, POS_REDEF_MENU_Y - OFFSET_Y_SELECT) : cpctm_screenPtr(CPCT_VMEM_START, POS_X_SELECT, POS_START_MENU_Y - OFFSET_Y_SELECT);
    cpct_drawSolidBox(memVideo, cpctm_px2byteM0(COLOR_BLACK, COLOR_BLACK), FLAMES_0_W, FLAMES_0_H);
}

/*****************************************************/
/*													 */
/*	Draw menu selection         		 			 */
/*												     */
/*****************************************************/
void DrawMenuSelection(BOOL selection, u8 tempoFlame)
{
    u8* memVideo = selection ? cpctm_screenPtr(CPCT_VMEM_START, POS_X_SELECT, POS_START_MENU_Y - OFFSET_Y_SELECT) : cpctm_screenPtr(CPCT_VMEM_START, POS_X_SELECT, POS_REDEF_MENU_Y - OFFSET_Y_SELECT);
    cpct_drawSprite(flames_tiles[ModuloPowerTwo(tempoFlame, NB_FLAMES)], memVideo, FLAMES_0_W, FLAMES_0_H);
}

/*****************************************************/
/*													 */
/*	Read key               				 			 */
/*												     */
/*****************************************************/
u8 ReadKey()
{
    cpct_scanKeyboard_i();
    return CheckUserInput();
}

/*****************************************************/
/*													 */
/*	Draw icon              				 			 */
/*												     */
/*****************************************************/
void DrawIcon(u8 iconId, u8 posX)
{
    cpct_drawSprite(_temp + iconId * ICON_SZ, GetScreenPtr(posX, POS_CREDITS_Y), ICON_CX, ICON_CY);
}

/*****************************************************/
/*													 */
/*	Draw credit text and icons			 			 */
/*												     */
/*****************************************************/
void PrintCreditIcon(u8 textLength, const char* text)
{
    u8 posText = POS_TEXT_X(textLength);
    PrintCredit(posText, text);
    DrawIcon(0, posText - ICON_CX - 1);
    DrawIcon(0, posText + textLength * FONT_MENU_CX);
}

/*****************************************************/
/*													 */
/*	Run menu               				 			 */
/*												     */
/*****************************************************/
void RunMenu()
{
    #define POS_X_CREDIT_5  POS_TEXT_X(27)
    #define OFFSET_LOGO_2   27
    #define OFFSET_LOGO_3   51

    BOOL startMenu = TRUE;
    u8 credit = 0;
    u8 wait = 50;

#ifdef WINCPCTELERA
    wait = 5;
#endif

    InitMenu();    

    while (TRUE)
    {        
        cpct_waitVSYNC();
        if (wait++ == 50)
        {
            u8 keyInput = ReadKey();            
            if (keyInput == FIRE)
            {
                if (startMenu)
                    return;

                RedefineKeys();
                DrawMenu();

                credit = 0;
                startMenu = TRUE;
            }
            else if (keyInput != NOTHING)
            {
                startMenu = startMenu ? FALSE : TRUE;
                CleanMenuSelection(startMenu);

                while (ReadKey() != NOTHING);
            }

            DrawMenuSelection(startMenu, credit);

            if (credit == 0)
                PrintCreditIcon(24, "A GAME BY ARNAUD BOUCHE");
            else
            if (credit == 50)
                PrintCreditIcon(24, "GRAPHICS BY CED AND ABO");
            else
            if (credit == 100)
                PrintCreditIcon(29, "INTRO BY KUKULCAN AND TITAN");
            else
            if (credit == 150)            
                PrintCreditIcon(14, "TESTED BY SLT");
            else
            if (credit == 200)
            {
                PrintCredit(POS_X_CREDIT_5, "USE CPCTELERA   AND ARKOS");
                DrawIcon(1, POS_X_CREDIT_5 + OFFSET_LOGO_2);
                DrawIcon(2, POS_X_CREDIT_5 + OFFSET_LOGO_3);
            }

            credit++;
            wait = 0;
        }
    }
}