//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------
#include <cpctelera.h>

/** Sprite includes */
#include "../sprites/local/redif_keys.h"

/** Local defines */
#define OFFSET_Y			15
#define POS_KEY_X			6

#define POS_REDEF_X			(SCREEN_CX - LOGO_KEY_CX) / 2
#define POS_REDEF_TITLE_Y	(POS_TITLE_Y + 30)
#define POS_REDEF_Y			(POS_REDEF_TITLE_Y + 10)

#define LOGO_KEY_CX			5
#define LOGO_KEY_CY			7
#define LOGO_KEY_SZ			(LOGO_KEY_CX*LOGO_KEY_CY)

/** External functions */
extern void DrawTitle();

/** External variables */
extern const cpct_keyID gKeys[NB_KEYS];

/*****************************************************/
/*													 */
/*	Get current key presses							 */
/*												     */
/*****************************************************/
cpct_keyID GetKeyPressed()
{
	/** From Ronaldo in cpcwiki forum */
	#define KEY_STATUS_SIZE		10

	u8* keyStatus = cpct_keyboardStatusBuffer;
	u8 i;

	while (!cpct_isAnyKeyPressed())
		cpct_scanKeyboard_i();

	for (i = 0; i < KEY_STATUS_SIZE; i++)
	{
		cpct_keyID keypressed = *keyStatus++ ^ 0xFF;
		if (keypressed)
			return (keypressed << 8) + i;
	}

	return 0;
}

/*****************************************************/
/*													 */
/*	Redefine a action key							 */
/*												     */
/*****************************************************/
void RedefineKey(u8 idKey)
{
	BOOL keyOk;
	cpct_keyID* defKey = (cpct_keyID*)&gKeys[idKey];
	cpct_keyID key;

	do
	{
		keyOk = TRUE;
		
  		key = GetKeyPressed();
		while (cpct_isKeyPressed(key))
			cpct_scanKeyboard_i();
		
		if (key != Key_Esc)
		{			
			if (idKey > 0)
			{
				u8 i;
				for (i = 0; i < idKey; i++)
				{
					if (gKeys[i] == key)
						keyOk = FALSE;
				}
			}
		}
		else
			keyOk = FALSE;

	} while (!keyOk);
	
	*defKey = key;
}

/*****************************************************/
/*													 */
/*	Draw key to redefine							 */
/*												     */
/*****************************************************/
void DrawRedefineKey(u8 y, u8 key)
{
	u8* vmem = GetScreenPtr(POS_REDEF_X, y);

	ZX7Decrunch(G_redif_keys_tileset[key], _temp);
	cpct_drawSprite(_temp, vmem, LOGO_KEY_CX, LOGO_KEY_CY);

	RedefineKey(key);
}

/*****************************************************/
/*													 */
/*	RedefinesKeys									 */
/*												     */
/*****************************************************/
void RedefineKeys()
{
	#define POS_TITLE_X		POS_TEXT_X(11)

	#define NB_KEYS		6
	u8 key;
	
	DrawTitle();

	ZX7Decrunch(G_letters, _temp);
	PrintStr(POS_TITLE_X, POS_REDEF_TITLE_Y, REDEF_KEYS_STR);
	cpct_drawSprite(_temp + LETTER_SZ, cpctm_screenPtr(CPCT_VMEM_START, POS_TITLE_X - LETTER_CX, POS_REDEF_TITLE_Y - OFFSET_Y_LETTER), LETTER_CX, LETTER_CY);

	while (cpct_isAnyKeyPressed()) 
		cpct_scanKeyboard_i();

	/** Draw keys to define */
	for (key = 0; key < NB_KEYS; key++)
		DrawRedefineKey(key * OFFSET_Y + (POS_REDEF_Y + OFFSET_Y), key);
}

