//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>

/** Game includes */
#include "../structures/structures.h"
#include "../text/text.h"
#include "../sound/sound.h"
#include "../video/video.h"
#include "../input/input.h"
#include "../player/player.h"
#include "../room/room.h"
#include "../world/world.h"
#include "../player/playerMove.h"

/** Sprites includes */
#include "../sprites/functions/packedSprites.h"
#include "../sprites/allSprites.h"

/** Defines */
#define POS_CHEST_X		0
#define POS_CHEST_Y		0

#define POS_ITEM_X		POS_CHEST_X + 2
#define POS_ITEM_Y		POS_CHEST_Y + 7

#define POS_SEL_X		POS_CHEST_X + (CHEST_UI_W * 2 - UI_SELECTION_W) / 2
#define POS_SEL_Y		POS_CHEST_Y + CHEST_UI_H

#define BACKGROUND_CX	CHEST_UI_W * 2
#define BACKGROUND_CY	(CHEST_UI_H + UI_SELECTION_H)

#define TEMP_BACKGROUND	(GET_TEMP_LOC_END - BACKGROUND_CX * BACKGROUND_CY)

/*****************************************************/
/*													 */
/*	Close Chest						  				 */
/*												     */
/*****************************************************/
void CloseChest()
{
    /** Restore background */
    u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_CHEST_X, POS_CHEST_Y);
    cpct_drawSprite(TEMP_BACKGROUND, vmem, BACKGROUND_CX, BACKGROUND_CY);

    /** Player Crouch Up */
    SetMovePlayer(TOP);
}

/*****************************************************/
/*													 */
/*	Chest action					  				 */
/*												     */
/*****************************************************/
void ChestAction()
{
    u8 move = CheckUserInput();
    u8 prevMove = NOTHING;

    u8* itemPtr = GetRoomChest()->items;
    u8 i;

    if (move == FIRE)
        WaitForFireReleased();

    move = NOTHING;
    while (move != FIRE)
    {
        move = CheckUserInput();
        if (move != NOTHING && move != prevMove)
            prevMove = move;
    } 
    
    for (i = 0; i < MAX_ITEMS_CHEST; i++)
        InteractWithItem(itemPtr++, CHEST_ACTION);

    PlaySelect(); 
}

/*****************************************************/
/*													 */
/*	Draw Chest Content				  				 */
/*												     */
/*****************************************************/
void DrawChestContent()
{
    /** Draw chest content */
    u8* itemPtr = GetRoomChest()->items;
    u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_ITEM_X, POS_ITEM_Y);

    u8 i;
    for (i = 0; i < MAX_ITEMS_CHEST; i++)
    {
        u8 itemId = *itemPtr++;
        if (itemId != NOTHING)
            DrawPackedSpriteMaskedFlip(GetObjectSpriteId(itemId), vmem + i * OBJECTS_W, OBJECTS_W, OBJECTS_H, FALSE);
    }

    vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_SEL_X, POS_SEL_Y);
    DrawPackedSpriteMaskedFlip(G_UI_SELECTION_0_INDEX, vmem, UI_SELECTION_W, UI_SELECTION_H, FALSE);
}

/*****************************************************/
/*													 */
/*	Draw Chest						  				 */
/*												     */
/*****************************************************/
void DrawChest()
{
    /** Save background */
    u8* vmem = cpctm_screenPtr(CPCT_VMEM_START, POS_CHEST_X, POS_CHEST_Y);
    cpct_getScreenToSprite(vmem, TEMP_BACKGROUND, BACKGROUND_CX, BACKGROUND_CY);
    
    /** Draw chest */
    DrawMirrorPackedSpriteMasked(G_CHEST_UI_INDEX, vmem, CHEST_UI_W, CHEST_UI_H);

    /** Draw chest content */
    DrawChestContent();

    /** Take objects */
    ChestAction();

    /** Exit */
    CloseChest();
}