//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>

/** Structures includes */
#include "../structures/structures.h"

/** Game includes */
#include "../tools/tools.h"

/** Asm includes */
#include "../asm/shiftSpritePixel.h"
#include "../asm/ZX7Decrunch.h"

/** Video include */
#include "video.h"

/** Create mask table */
cpctm_createTransparentMaskTable(gMaskTable, MASK_TABLE_LOC, M0, 0);

/*****************************************************/
/*													 */
/*	Get video buffer pointer						 */
/*												     */
/*****************************************************/
u8* GetScreenPtr(u8 xPos, u8 yPos)
{
    return cpct_getScreenPtr(CPCT_VMEM_START, xPos, yPos);
}

/*****************************************************/
/*													 */
/*	Flip and shift sprite							 */
/*												     */
/*****************************************************/
void FlipAndShiftSprite(u8* sprite, u8 width, u8 height, u8 shift)
{
    cpct_hflipSpriteM0(width, height, sprite);

    if (shift == LEFT)
        asm_shiftSpritePixelsLeft(sprite, width * height);
    else
        asm_shiftSpritePixelsRight(sprite, width * height);
}

/*****************************************************/
/*													 */
/*	Get decompress flipped sprites					 */
/*												     */
/*****************************************************/
u8* GetDecompressSpriteFlip(u8* data, u8 cx, u8 cy, BOOL flip)
{
    u8* sprite = Decompress(data);

    if (flip)
        cpct_hflipSpriteM0(cx, cy, sprite);

    return sprite;
}

/*****************************************************/
/*													 */
/*	Decompress and draw sprites						 */
/*												     */
/*****************************************************/
void DrawCompressedSprite(u8* data, u8* vmem, u8 cx, u8 cy, BOOL flip)
{
    u8* sprite = GetDecompressSpriteFlip(data, cx, cy, flip);
    cpct_drawSprite(sprite, vmem, cx, cy);
}

/*****************************************************/
/*													 */
/*	Draw sprites masked								 */
/*												     */
/*****************************************************/
void DrawSpriteMasked(u8* sprite, u8* vmem, u8 cx, u8 cy)
{
    cpct_drawSpriteMaskedAlignedTable(sprite, vmem, cx, cy, gMaskTable);
}

/*****************************************************/
/*													 */
/*	Decompress and draw sprites	masked				 */
/*												     */
/*****************************************************/
void DrawCompressedSpriteMasked(u8* data, u8* vmem, u8 cx, u8 cy, BOOL flip)
{
    u8* sprite = GetDecompressSpriteFlip(data, cx, cy, flip);
    cpct_drawSpriteMaskedAlignedTable(sprite, vmem, cx, cy, gMaskTable);
}

/*****************************************************/
/*													 */
/*	Decompress and draw mirror sprites masked		 */
/*												     */
/*****************************************************/
void DrawMirrorCompressedSpriteMasked(u8* data, u8* vmem, u8 cx, u8 cy)
{
    u8* sprite = Decompress(data);
    cpct_drawSpriteMaskedAlignedTable(sprite, vmem, cx, cy, gMaskTable);

    cpct_hflipSpriteM0(cx, cy, sprite);
    cpct_drawSpriteMaskedAlignedTable(sprite, vmem + cx, cx, cy, gMaskTable);
}

/*****************************************************/
/*													 */
/*	End memory                              		 */
/*												     */
/*****************************************************/
void dummy_end_code()
{
}