//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Throne Legacy
//  Copyright (C) 2020 Arnaud Bouche (@Arnaud6128)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

/** CPCTelera includes */
#include <cpctelera.h>
#include <stdio.h>

/** ZX7 includes */
#include "../asm/ZX7Decrunch.h"

/** Structures includes */
#include "../structures/memMapping.h"
#include "../structures/structures.h"

/** Game includes */
#include "../video/video.h"
#include "../player/player.h"
#include "../room/room.h"
#include "../tools/tools.h"
#include "../sprites/allSprites.h"

/** World includes */
#include "./world.h"

/** External World definition */
extern SWorld* gWorld;
extern SWorldStatic* gWorldStatic;
extern u8 gRoomId;
extern u8 gAdventure;
extern u8 gColorSky;

/*****************************************************/
/*												     */
/*	Reset world 									 */
/*												     */
/*****************************************************/
void ResetWorld()
{
    gColorSky = SKY_COLOR;

#ifndef WINCPCTELERA
    ZX7Decrunch(GET_WORLD_LOC_COMPRESS, (u8*)gWorld);
#endif
}

/*****************************************************/
/*												     */
/*	Is game win 									 */
/*												     */
/*****************************************************/
BOOL IsGameWin()
{
    return IS_GAME_WIN(gAdventure);
}

/*****************************************************/
/*												     */
/*	Get Room										 */
/*												     */
/*****************************************************/
SRoom* GetRoom(u8 roomId)
{
    SetRandSeed(roomId);
    return &gWorld->rooms[roomId];
}

/*****************************************************/
/*												     */
/*	Have Door										 */
/*												     */
/*****************************************************/
BOOL HaveDoor(u8 door)
{
    u8 doorType = GET_DOOR_TYPE(door);
    return (doorType != DOOR_TYPE_WALL && doorType != DOOR_TYPE_NO_WALL);
}

/*****************************************************/
/*												     */
/*	Get Chest										 */
/*												     */
/*****************************************************/
SChest* GetRoomChest()
{
    SChest* chest = gWorld->chests;

    u8 i;
    for (i = 0; i < MAX_CHEST; i++)
    {
        if (chest->roomId == gRoomId)
            return chest;

        chest++;
    }

    return NULL;
}

/*****************************************************/
/*												     */
/*	Get Room items  								 */
/*												     */
/*****************************************************/
SItemsRoom* GetRoomItems(u8 roomId)
{
    SItemsRoom* itemRooms = gWorld->roomsItems;

    u8 i;
    for (i = 0; i < MAX_ROOMS_ITEMS; i++)
    {
        if (itemRooms->roomId == roomId)
            return itemRooms;

        itemRooms++;
    }

    return NULL;
}

/*****************************************************/
/*												     */
/*	Get Object spriteId								 */
/*												     */
/*****************************************************/
u16 GetObjectSpriteId(u8 objectId)
{
    if (objectId > SCROLL)
        objectId = SCROLL;

    return G_objects_tileset[objectId];
}

